##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" CMFDefault portal_registration tool.

$Id: RegistrationTool.py 77182 2007-06-28 17:25:27Z yuppie $
"""

from warnings import warn

from AccessControl import ClassSecurityInfo
from AccessControl.requestmethod import postonly
from Acquisition import aq_base
from Globals import InitializeClass
from Products.MailHost.interfaces import IMailHost
from zope.component import getUtility
from zope.schema import ValidationError

from Products.CMFCore.interfaces import IRegistrationTool
from Products.CMFCore.RegistrationTool import RegistrationTool as BaseTool
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import getToolByName

from permissions import ManagePortal
from utils import checkEmailAddress
from utils import Message as _


class RegistrationTool(BaseTool):

    """ Manage through-the-web signup policies.
    """

    __implements__ = BaseTool.__implements__

    meta_type = 'Default Registration Tool'

    security = ClassSecurityInfo()

    def _getValidEmailAddress(self, member):
        email = member.getProperty('email')

        # assert that we can actually get an email address, otherwise
        # the template will be made with a blank To:, this is bad
        if email is None:
            msg = _(u'No email address is registered for member: '
                    u'${member_id}', mapping={'member_id': member.getId()})
            raise ValueError(msg)

        checkEmailAddress(email)
        return email

    #
    #   'portal_registration' interface
    #
    security.declarePublic( 'testPasswordValidity' )
    def testPasswordValidity(self, password, confirm=None):

        """ Verify that the password satisfies the portal's requirements.

        o If the password is valid, return None.
        o If not, return a string explaining why.
        """
        if not password:
            return _(u'You must enter a password.')

        if len(password) < 5 and not _checkPermission(ManagePortal, self):
            return _(u'Your password must contain at least 5 characters.')

        if confirm is not None and confirm != password:
            return _(u'Your password and confirmation did not match. '
                     u'Please try again.')

        return None

    security.declarePublic( 'testPropertiesValidity' )
    def testPropertiesValidity(self, props, member=None):

        """ Verify that the properties supplied satisfy portal's requirements.

        o If the properties are valid, return None.
        o If not, return a string explaining why.
        """
        if member is None: # New member.

            username = props.get('username', '')
            if not username:
                return _(u'You must enter a valid name.')

            if not self.isMemberIdAllowed(username):
                return _(u'The login name you selected is already in use or '
                         u'is not valid. Please choose another.')

            email = props.get('email')
            if email is None:
                return _(u'You must enter an email address.')

            try:
                checkEmailAddress(email)
            except ValidationError:
                return _(u'You must enter a valid email address.')

        else: # Existing member.
            email = props.get('email')

            if email is not None:
                try:
                    checkEmailAddress(email)
                except ValidationError:
                    return _(u'You must enter a valid email address.')

            # Not allowed to clear an existing non-empty email.
            existing = member.getProperty('email')

            if existing and email == '':
                return _(u'You must enter a valid email address.')

        return None

    security.declarePublic( 'mailPassword' )
    def mailPassword(self, forgotten_userid, REQUEST):
        """ Email a forgotten password to a member.

        o Raise an exception if user ID is not found.
        """
        # XXX: this method violates the rules for tools/utilities:
        # it depends on a non-utility tool
        membership = getToolByName(self, 'portal_membership')
        member = membership.getMemberById(forgotten_userid)

        if member is None:
            raise ValueError(_(u'The username you entered could not be '
                               u'found.'))

        email = self._getValidEmailAddress(member)

        # Rather than have the template try to use the mailhost, we will
        # render the message ourselves and send it from here (where we
        # don't need to worry about 'UseMailHost' permissions).
        method = self.password_email
        kw = {'member': member, 'password': member.getPassword()}

        if getattr(aq_base(method), 'isDocTemp', 0):
            mail_text = method(self, REQUEST, **kw)
        else:
            mail_text = method(**kw)

        host = getUtility(IMailHost)
        host.send( mail_text )

        return self.mail_password_response( self, REQUEST )

    security.declarePublic( 'registeredNotify' )
    def registeredNotify(self, new_member_id, password=None, REQUEST=None):
        """ Handle mailing the registration / welcome message.
        """
        # XXX: this method violates the rules for tools/utilities:
        # it depends on a non-utility tool and uses self.REQUEST
        if REQUEST is None:
            REQUEST = self.REQUEST
            warn("registeredNotify should be called with 'REQUEST' as third "
                 "argument. The BBB code will be removed in CMF 2.3.",
                 DeprecationWarning, stacklevel=2)

        membership = getToolByName( self, 'portal_membership' )
        member = membership.getMemberById( new_member_id )

        if member is None:
            raise ValueError(_(u'The username you entered could not be '
                               u'found.'))

        if password is None:
            password = member.getPassword()

        email = self._getValidEmailAddress(member)

        # Rather than have the template try to use the mailhost, we will
        # render the message ourselves and send it from here (where we
        # don't need to worry about 'UseMailHost' permissions).
        method = self.registered_email
        kw = {'member': member, 'password': password, 'email': email}

        if getattr(aq_base(method), 'isDocTemp', 0):
            mail_text = method(self, REQUEST, **kw)
        else:
            mail_text = method(**kw)

        host = getUtility(IMailHost)
        host.send( mail_text )

    security.declareProtected(ManagePortal, 'editMember')
    @postonly
    def editMember(self, member_id, properties=None, password=None,
                   roles=None, domains=None, REQUEST=None):
        """ Edit a user's properties and security settings

        o Checks should be done before this method is called using
          testPropertiesValidity and testPasswordValidity
        """
        # XXX: this method violates the rules for tools/utilities:
        # it depends on a non-utility tool
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getMemberById(member_id)
        member.setMemberProperties(properties)
        member.setSecurityProfile(password,roles,domains)

        return member

InitializeClass(RegistrationTool)



##parameters=id='', **kw
##
from Products.CMFDefault.utils import Message as _

if id:
    if context.checkIdAvailable(id):
        return context.setStatus(True)
    else:
        return context.setStatus(False, _(u'Please choose another ID.'))
else:
    return context.setStatus(False, _(u'Please enter an ID.'))



##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser views for folders.

$Id: folder.py 77366 2007-07-03 16:14:33Z yuppie $
"""

from DocumentTemplate import sequence
from Products.PythonScripts.standard import thousands_commas
from ZTUtils import Batch
from ZTUtils import LazyFilter
from ZTUtils import make_query

from Products.CMFCore.interfaces import IDynamicType
from Products.CMFDefault.exceptions import CopyError
from Products.CMFDefault.exceptions import zExceptions_Unauthorized
from Products.CMFDefault.permissions import AddPortalContent
from Products.CMFDefault.permissions import DeleteObjects
from Products.CMFDefault.permissions import ListFolderContents
from Products.CMFDefault.permissions import ManageProperties
from Products.CMFDefault.permissions import ViewManagementScreens
from Products.CMFDefault.utils import html_marshal
from Products.CMFDefault.utils import translate
from Products.CMFDefault.utils import Message as _

from utils import decode
from utils import memoize
from utils import ViewBase


# XXX: This should be refactored using formlib. Please don't import from this
#      module, things might be changed without further notice.

class FormViewBase(ViewBase):

    # helpers

    def _setRedirect(self, provider_id, action_path, keys=''):
        provider = self._getTool(provider_id)
        try:
            target = provider.getActionInfo(action_path, self.context)['url']
        except ValueError:
            target = self._getPortalURL()

        kw = {}
        message = self.request.other.get('portal_status_message', '')
        if message:
            if isinstance(message, unicode):
                message = message.encode(self._getBrowserCharset())
            kw['portal_status_message'] = message
        for k in keys.split(','):
            k = k.strip()
            v = self.request.form.get(k, None)
            if v:
                kw[k] = v

        query = kw and ( '?%s' % make_query(kw) ) or ''
        self.request.RESPONSE.redirect( '%s%s' % (target, query) )

        return True

    # interface

    def __call__(self, **kw):
        form = self.request.form
        for button in self._BUTTONS:
            if button['id'] in form:
                for permission in button.get('permissions', ()):
                    if not self._checkPermission(permission):
                        break
                else:
                    for transform in button.get('transform', ()):
                        status = getattr(self, transform)(**form)
                        if isinstance(status, bool):
                            status = (status,)
                        if len(status) > 1:
                            message = translate(status[1], self.context)
                            self.request.other['portal_status_message'] = message
                        if not status[0]:
                            return self.index()
                    if self._setRedirect(*button['redirect']):
                        return
        return self.index()

    @memoize
    def form_action(self):
        return self._getViewURL()

    @memoize
    def listButtonInfos(self):
        form = self.request.form
        buttons = []
        for button in self._BUTTONS:
            if button.get('title', None):
                for permission in button.get('permissions', ()):
                    if not self._checkPermission(permission):
                        break
                else:
                    for condition in button.get('conditions', ()):
                        if not getattr(self, condition)():
                            break
                    else:
                        buttons.append({'name': button['id'],
                                        'value': button['title']})
        return tuple(buttons)

    @memoize
    @decode
    def listHiddenVarInfos(self):
        kw = self._getHiddenVars()
        vars = [ {'name': name, 'value': value}
                 for name, value in html_marshal(**kw) ]
        return tuple(vars)


class BatchViewBase(ViewBase):

    # helpers

    _BATCH_SIZE = 25

    @memoize
    def _getBatchStart(self):
        return self.request.form.get('b_start', 0)

    @memoize
    def _getBatchObj(self):
        b_start = self._getBatchStart()
        items = self._getItems()
        return Batch(items, self._BATCH_SIZE, b_start, orphan=0)

    @memoize
    def _getHiddenVars(self):
        return {}

    @memoize
    def _getNavigationVars(self):
        return self._getHiddenVars()

    @memoize
    def _getNavigationURL(self, b_start):
        target = self._getViewURL()
        kw = self._getNavigationVars().copy()

        kw['b_start'] = b_start
        for k, v in kw.items():
            if not v or k == 'portal_status_message':
                del kw[k]

        query = kw and ('?%s' % make_query(kw)) or ''
        return u'%s%s' % (target, query)

    # interface

    @memoize
    @decode
    def listItemInfos(self):
        batch_obj = self._getBatchObj()
        portal_url = self._getPortalURL()

        items = []
        for item in batch_obj:
            item_description = item.Description()
            item_icon = item.getIcon(1)
            item_title = item.Title()
            item_type = remote_type = item.Type()
            if item_type == 'Favorite' and not item_icon == 'p_/broken':
                item = item.getObject()
                item_description = item_description or item.Description()
                item_title = item_title or item.Title()
                remote_type = item.Type()
            is_file = remote_type in ('File', 'Image')
            is_link = remote_type == 'Link'
            items.append({'description': item_description,
                          'format': is_file and item.Format() or '',
                          'icon': item_icon and ('%s/%s' %
                                               (portal_url, item_icon)) or '',
                          'size': is_file and ('%0.0f kb' %
                                            (item.get_size() / 1024.0)) or '',
                          'title': item_title,
                          'type': item_type,
                          'url': is_link and item.getRemoteUrl() or
                                 item.absolute_url()})
        return tuple(items)

    @memoize
    def navigation_previous(self):
        batch_obj = self._getBatchObj().previous
        if batch_obj is None:
            return None

        length = len(batch_obj)
        url = self._getNavigationURL(batch_obj.first)
        if length == 1:
            title = _(u'Previous item')
        else:
            title = _(u'Previous ${count} items', mapping={'count': length})
        return {'title': title, 'url': url}

    @memoize
    def navigation_next(self):
        batch_obj = self._getBatchObj().next
        if batch_obj is None:
            return None

        length = len(batch_obj)
        url = self._getNavigationURL(batch_obj.first)
        if length == 1:
            title = _(u'Next item')
        else:
            title = _(u'Next ${count} items', mapping={'count': length})
        return {'title': title, 'url': url}

    @memoize
    def summary_length(self):
        length = self._getBatchObj().sequence_length
        return length and thousands_commas(length) or ''

    @memoize
    def summary_type(self):
        length = self._getBatchObj().sequence_length
        return (length == 1) and _(u'item') or _(u'items')

    @memoize
    @decode
    def summary_match(self):
        return self.request.form.get('SearchableText')


class FolderView(BatchViewBase):

    """View for IFolderish.
    """

    # helpers

    @memoize
    def _getItems(self):
        (key, reverse) = self.context.getDefaultSorting()
        items = self.context.contentValues()
        items = sequence.sort(items,
                              ((key, 'cmp', reverse and 'desc' or 'asc'),))
        return LazyFilter(items, skip='View')

    # interface

    @memoize
    def has_local(self):
        return 'local_pt' in self.context.objectIds()


class FolderContentsView(BatchViewBase, FormViewBase):

    """Contents view for IFolderish.
    """

    _BUTTONS = ({'id': 'items_new',
                 'title': _(u'New...'),
                 'permissions': (ViewManagementScreens, AddPortalContent),
                 'conditions': ('checkAllowedContentTypes',),
                 'redirect': ('portal_types', 'object/new')},
                {'id': 'items_rename',
                 'title': _(u'Rename...'),
                 'permissions': (ViewManagementScreens, AddPortalContent),
                 'conditions': ('checkItems', 'checkAllowedContentTypes'),
                 'transform': ('validateItemIds',),
                 'redirect': ('portal_types', 'object/rename_items',
                              'b_start, ids, key, reverse')},
                {'id': 'items_cut',
                 'title': _(u'Cut'),
                 'permissions': (ViewManagementScreens,),
                 'conditions': ('checkItems',),
                 'transform': ('validateItemIds', 'cut_control'),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start, key, reverse')},
                {'id': 'items_copy',
                 'title': _(u'Copy'),
                 'permissions': (ViewManagementScreens,),
                 'conditions': ('checkItems',),
                 'transform': ('validateItemIds', 'copy_control'),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start, key, reverse')},
                {'id': 'items_paste',
                 'title': _(u'Paste'),
                 'permissions': (ViewManagementScreens, AddPortalContent),
                 'conditions': ('checkClipboardData',),
                 'transform': ('validateClipboardData', 'paste_control'),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start, key, reverse')},
                {'id': 'items_delete',
                 'title': _(u'Delete'),
                 'permissions': (ViewManagementScreens, DeleteObjects),
                 'conditions': ('checkItems',),
                 'transform': ('validateItemIds', 'delete_control'),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start, key, reverse')},
                {'id': 'items_sort',
                 'permissions': (ManageProperties,),
                 'transform': ('sort_control',),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start')},
                {'id': 'items_up',
                 'permissions': (ManageProperties,),
                 'transform': ('validateItemIds', 'up_control'),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start, key, reverse')},
                {'id': 'items_down',
                 'permissions': (ManageProperties,),
                 'transform': ('validateItemIds', 'down_control'),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start, key, reverse')},
                {'id': 'items_top',
                 'permissions': (ManageProperties,),
                 'transform': ('validateItemIds', 'top_control'),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start, key, reverse')},
                {'id': 'items_bottom',
                 'permissions': (ManageProperties,),
                 'transform': ('validateItemIds', 'bottom_control'),
                 'redirect': ('portal_types', 'object/folderContents',
                              'b_start, key, reverse')},
                {'id': 'set_view_filter',
                 'transform': ('set_filter_control',),
                 'redirect': ('portal_types', 'object/folderContents')},
                {'id': 'clear_view_filter',
                 'transform': ('clear_filter_control',),
                 'redirect': ('portal_types', 'object/folderContents')})

    # helpers

    @memoize
    def _getSorting(self):
        key = self.request.form.get('key', None)
        if key:
            return (key, self.request.form.get('reverse', 0))
        else:
            return self.context.getDefaultSorting()

    @memoize
    def _isDefaultSorting(self):
        return self._getSorting() == self.context.getDefaultSorting()

    @memoize
    def _getHiddenVars(self):
        b_start = self._getBatchStart()
        is_default = self._isDefaultSorting()
        (key, reverse) = is_default and ('', 0) or self._getSorting()
        return {'b_start': b_start, 'key': key, 'reverse': reverse}

    @memoize
    def _getItems(self):
        (key, reverse) = self._getSorting()
        folderfilter = self.request.get('folderfilter', '')
        filter = self.context.decodeFolderFilter(folderfilter)
        items = self.context.listFolderContents(contentFilter=filter)
        return sequence.sort(items,
                             ((key, 'cmp', reverse and 'desc' or 'asc'),))

    # interface

    @memoize
    @decode
    def up_info(self):
        up_obj = self.context.aq_inner.aq_parent
        mtool = self._getTool('portal_membership')
        allowed = mtool.checkPermission(ListFolderContents, up_obj)
        if allowed:
            if IDynamicType.providedBy(up_obj):
                up_url = up_obj.getActionInfo('object/folderContents')['url']
                return {'icon': '%s/UpFolder_icon.gif' % self._getPortalURL(),
                        'id': up_obj.getId(),
                        'url': up_url}
            else:
                return {'icon': '',
                        'id': 'Root',
                        'url': ''}
        else:
            return {}

    @memoize
    def listColumnInfos(self):
        (key, reverse) = self._getSorting()
        columns = ( {'key': 'Type',
                     'title': _(u'Type'),
                     'width': '20',
                     'colspan': '2'}
                  , {'key': 'getId',
                     'title': _(u'Name'),
                     'width': '360',
                     'colspan': None}
                  , {'key': 'modified',
                     'title': _(u'Last Modified'),
                     'width': '180',
                     'colspan': None}
                  , {'key': 'position',
                     'title': _(u'Position'),
                     'width': '80',
                     'colspan': None }
                  )
        for column in columns:
            if key == column['key'] and not reverse and key != 'position':
                query = make_query(key=column['key'], reverse=1)
            else:
                query = make_query(key=column['key'])
            column['url'] = '%s?%s' % (self._getViewURL(), query)
        return tuple(columns)

    @memoize
    @decode
    def listItemInfos(self):
        b_start = self._getBatchStart()
        (key, reverse) = self._getSorting()
        batch_obj = self._getBatchObj()
        items_manage_allowed = self._checkPermission(ViewManagementScreens)
        portal_url = self._getPortalURL()

        items = []
        i = 1
        for item in batch_obj:
            item_icon = item.getIcon(1)
            item_id = item.getId()
            item_position = (key == 'position') and str(b_start + i) or '...'
            i += 1
            item_url = item.getActionInfo(('object/folderContents',
                                           'object/view'))['url']
            items.append({'checkbox': items_manage_allowed and ('cb_%s' %
                                                               item_id) or '',
                          'icon': item_icon and ('%s/%s' %
                                               (portal_url, item_icon)) or '',
                          'id': item_id,
                          'modified': item.ModificationDate(),
                          'position': item_position,
                          'title': item.Title(),
                          'type': item.Type() or None,
                          'url': item_url})
        return tuple(items)

    @memoize
    def listDeltas(self):
        length = self._getBatchObj().sequence_length
        deltas = range(1, min(5, length)) + range(5, length, 5)
        return tuple(deltas)

    @memoize
    def is_orderable(self):
        length = len(self._getBatchObj())
        items_move_allowed = self._checkPermission(ManageProperties)
        (key, reverse) = self._getSorting()
        return items_move_allowed and (key == 'position') and length > 1

    @memoize
    def is_sortable(self):
        items_move_allowed = self._checkPermission(ManageProperties)
        return items_move_allowed and not self._isDefaultSorting()

    # checkers

    def checkAllowedContentTypes(self):
        return bool(self.context.allowedContentTypes())

    def checkClipboardData(self):
        return bool(self.context.cb_dataValid())

    def checkItems(self):
        return bool(self._getItems())

    # validators

    def validateItemIds(self, ids=(), **kw):
        if ids:
            return True
        else:
            return False, _(u'Please select one or more items first.')

    def validateClipboardData(self, **kw):
        if self.context.cb_dataValid():
            return True
        else:
            return False, _(u'Please copy or cut one or more items to paste '
                            u'first.')

    # controllers

    def cut_control(self, ids, **kw):
        """Cut objects from a folder and copy to the clipboard.
        """
        try:
            self.context.manage_cutObjects(ids, self.request)
            if len(ids) == 1:
                return True, _(u'Item cut.')
            else:
                return True, _(u'Items cut.')
        except CopyError:
            return False, _(u'CopyError: Cut failed.')
        except zExceptions_Unauthorized:
            return False, _(u'Unauthorized: Cut failed.')

    def copy_control(self, ids, **kw):
        """Copy objects from a folder to the clipboard.
        """
        try:
            self.context.manage_copyObjects(ids, self.request)
            if len(ids) == 1:
                return True, _(u'Item copied.')
            else:
                return True, _(u'Items copied.')
        except CopyError:
            return False, _(u'CopyError: Copy failed.')

    def paste_control(self, **kw):
        """Paste objects to a folder from the clipboard.
        """
        try:
            result = self.context.manage_pasteObjects(self.request['__cp'])
            if len(result) == 1:
                return True, _(u'Item pasted.')
            else:
                return True, _(u'Items pasted.')
        except CopyError:
            return False, _(u'CopyError: Paste failed.')
        except zExceptions_Unauthorized:
            return False, _(u'Unauthorized: Paste failed.')

    def delete_control(self, ids, **kw):
        """Delete objects from a folder.
        """
        self.context.manage_delObjects(list(ids))
        if len(ids) == 1:
            return True, _(u'Item deleted.')
        else:
            return True, _(u'Items deleted.')

    def sort_control(self, key='position', reverse=0, **kw):
        """Sort objects in a folder.
        """
        self.context.setDefaultSorting(key, reverse)
        return True

    def up_control(self, ids, delta, **kw):
        subset_ids = [ obj.getId()
                       for obj in self.context.listFolderContents() ]
        try:
            attempt = self.context.moveObjectsUp(ids, delta,
                                                 subset_ids=subset_ids)
            if attempt == 1:
                return True, _(u'Item moved up.')
            elif attempt > 1:
                return True, _(u'Items moved up.')
            else:
                return False, _(u'Nothing to change.')
        except ValueError:
            return False, _(u'ValueError: Move failed.')

    def down_control(self, ids, delta, **kw):
        subset_ids = [ obj.getId()
                       for obj in self.context.listFolderContents() ]
        try:
            attempt = self.context.moveObjectsDown(ids, delta,
                                                   subset_ids=subset_ids)
            if attempt == 1:
                return True, _(u'Item moved down.')
            elif attempt > 1:
                return True, _(u'Items moved down.')
            else:
                return False, _(u'Nothing to change.')
        except ValueError:
            return False, _(u'ValueError: Move failed.')

    def top_control(self, ids, **kw):
        subset_ids = [ obj.getId()
                       for obj in self.context.listFolderContents() ]
        try:
            attempt = self.context.moveObjectsToTop(ids,
                                                    subset_ids=subset_ids)
            if attempt == 1:
                return True, _(u'Item moved to top.')
            elif attempt > 1:
                return True, _(u'Items moved to top.')
            else:
                return False, _(u'Nothing to change.')
        except ValueError:
            return False, _(u'ValueError: Move failed.')

    def bottom_control(self, ids, **kw):
        subset_ids = [ obj.getId()
                       for obj in self.context.listFolderContents() ]
        try:
            attempt = self.context.moveObjectsToBottom(ids,
                                                       subset_ids=subset_ids)
            if attempt == 1:
                return True, _(u'Item moved to bottom.')
            elif attempt > 1:
                return True, _(u'Items moved to bottom.')
            else:
                return False, _(u'Nothing to change.')
        except ValueError:
            return False, _(u'ValueError: Move failed.')

    def set_filter_control(self, **kw):
        filter = self.context.encodeFolderFilter(self.request)
        self.request.RESPONSE.setCookie('folderfilter', filter, path='/',
                                      expires='Wed, 19 Feb 2020 14:28:00 GMT')
        return True, _(u'Filter applied.')

    def clear_filter_control(self, **kw):
        self.request.RESPONSE.expireCookie('folderfilter', path='/')
        self.request.RESPONSE.expireCookie('show_filter_form', path='/')
        return True, _(u'Filter cleared.')



##parameters=
##
from ZTUtils import Batch
from ZTUtils import LazyFilter
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import decode

stool = getUtilityByInterfaceName('Products.CMFCore.interfaces.ISyndicationTool')


if not stool.isSyndicationAllowed(context):
    context.REQUEST.RESPONSE.redirect(context.absolute_url() +
             '/rssDisabled?portal_status_message=Syndication+is+Disabled')
    return


options = {}

options['channel_info'] = { 'base': stool.getHTML4UpdateBase(context),
                            'description': context.Description(),
                            'frequency': stool.getUpdateFrequency(context),
                            'period': stool.getUpdatePeriod(context),
                            'title': context.Title(),
                            'url': context.absolute_url() }

key, reverse = context.getDefaultSorting()
items = stool.getSyndicatableContent(context)
items = sequence.sort( items, ((key, 'cmp', reverse and 'desc' or 'asc'),) )
items = LazyFilter(items, skip='View')
b_size = stool.getMaxItems(context)
batch_obj = Batch(items, b_size, 0, orphan=0)
items = []
for item in batch_obj:
    items.append( { 'date': item.modified().HTML4(),
                    'description': item.Description(),
                    'listCreators': item.listCreators(),
                    'listSubjects': item.Subject(),
                    'publisher': item.Publisher(),
                    'rights': item.Rights(),
                    'title': item.Title(),
                    'url': item.absolute_url() } )
options['listItemInfos'] = tuple(items)

return context.RSS_template(**decode(options, script))



##parameters=text_format, text, SafetyBelt='', **kw
##
from Products.CMFDefault.exceptions import EditingConflict
from Products.CMFDefault.exceptions import ResourceLockedError
from Products.CMFDefault.utils import Message as _

if text_format != context.text_format or text != context.EditableBody():
    try:
        context.edit(text_format, text, safety_belt=SafetyBelt)
        return context.setStatus(True, _(u'Document changed.'))
    except (ResourceLockedError, EditingConflict), errmsg:
        return context.setStatus(False, errmsg)
else:
    return context.setStatus(False, _(u'Nothing to change.'))



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for NewsItem module.

$Id: test_NewsItem.py 68344 2006-05-29 14:19:25Z yuppie $
"""

import unittest
import Testing

from Products.CMFCore.testing import ConformsToContent
from Products.CMFCore.tests.base.content import BASIC_HTML
from Products.CMFCore.tests.base.content import BASIC_STRUCTUREDTEXT
from Products.CMFCore.tests.base.content import DOCTYPE
from Products.CMFCore.tests.base.content import ENTITY_IN_TITLE
from Products.CMFCore.tests.base.testcase import RequestTest


class NewsItemTests(ConformsToContent, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFDefault.NewsItem import NewsItem

        return NewsItem

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_z3interfaces(self):
        from zope.interface.verify import verifyClass
        from Products.CMFDefault.interfaces import IDocument
        from Products.CMFDefault.interfaces import IMutableDocument
        from Products.CMFDefault.interfaces import IMutableNewsItem
        from Products.CMFDefault.interfaces import INewsItem

        verifyClass(IDocument, self._getTargetClass())
        # XXX: different signature
        # verifyClass(IMutableDocument, self._getTargetClass())
        verifyClass(IMutableNewsItem, self._getTargetClass())
        verifyClass(INewsItem, self._getTargetClass())

    def test_Empty_html(self):
        d = self._makeOne('empty', text_format='html')

        self.assertEqual( d.Title(), '' )
        self.assertEqual( d.Description(), '' )
        self.assertEqual( d.Format(), 'text/html' )
        self.assertEqual( d.text_format, 'html' )
        self.assertEqual( d.text, '' )

    def test_Empty_stx(self):
        d = self._makeOne('foo', text_format='structured-text')

        self.assertEqual( d.Title(), '' )
        self.assertEqual( d.Description(), '' )
        self.assertEqual( d.Format(), 'text/plain' )
        self.assertEqual( d.text_format, 'structured-text' )
        self.assertEqual( d.text, '' )

    def test_Init_with_stx( self ):
        d = self._makeOne('foo', text_format='structured-text',
                          title='Foodoc')

        self.assertEqual( d.Title(), 'Foodoc' )
        self.assertEqual( d.Description(), '' )
        self.assertEqual( d.Format(), 'text/plain' )
        self.assertEqual( d.text_format, 'structured-text' )
        self.assertEqual( d.text, '' )

    def test_default_format( self ):
        d = self._makeOne('foo', text='')

        self.assertEqual( d.Format(), 'text/plain' )
        self.assertEqual( d.text_format, 'structured-text' )


class NewsItemPUTTests(RequestTest):

    def _getTargetClass(self):
        from Products.CMFDefault.NewsItem import NewsItem

        return NewsItem

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_Init(self):
        self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
        d = self._makeOne('foo', text='')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual( d.Title(), 'My Document' )
        self.assertEqual( d.Description(), 'A document by me' )
        self.assertEqual( d.Format(), 'text/plain' )
        self.assertEqual( d.text_format, 'structured-text' )
        self.assertEqual( len(d.Contributors()), 3 )
        self.failUnless( d.cooked_text.find('<p>') >= 0 )

    def test_PUT_basic_html(self):
        self.REQUEST['BODY']=BASIC_HTML
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual( d.Title(), 'Title in tag' )
        self.assertEqual( d.Description(), 'Describe me' )
        self.assertEqual( d.Format(), 'text/html' )
        self.assertEqual( d.text_format, 'html' )
        self.assertEqual( d.text.find('</body>'), -1 )
        self.assertEqual( len(d.Contributors()), 3 )

    def test_PUT_uppered_html(self):
        self.REQUEST['BODY'] = BASIC_HTML.upper()
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual( d.Title(), 'TITLE IN TAG' )
        self.assertEqual( d.Description(), 'DESCRIBE ME' )
        self.assertEqual( d.Format(), 'text/html' )
        self.assertEqual( d.text_format, 'html' )
        self.assertEqual( d.text.find('</BODY'), -1 )
        self.assertEqual( len(d.Contributors()), 3 )

    def test_PUT_entity_in_title(self):
        self.REQUEST['BODY'] = ENTITY_IN_TITLE
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual( d.Title(), '&Auuml;rger' )

    def test_PUT_html_with_doctype(self):
        self.REQUEST['BODY'] = '%s\n%s' % (DOCTYPE, BASIC_HTML)
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual( d.Description(), 'Describe me' )

    def test_PUT_html_without_newlines(self):
        self.REQUEST['BODY'] = ''.join(BASIC_HTML.split('\n'))
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual( d.Title(), 'Title in tag' )
        self.assertEqual( d.Description(), 'Describe me' )
        self.assertEqual( d.Format(), 'text/html' )
        self.assertEqual( d.text_format, 'html' )
        self.assertEqual( d.text.find('</body>'), -1 )
        self.assertEqual( len(d.Contributors()), 3 )

    def test_PUT_structured_text(self):
        self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
        d = self._makeOne('foo')
        d.PUT( self.REQUEST, self.RESPONSE )

        self.assertEqual( d.Title(), 'My Document')
        self.assertEqual( d.Description(), 'A document by me')
        self.assertEqual( d.Format(), 'text/plain' )
        self.assertEqual( d.text_format, 'structured-text' )
        self.assertEqual( len(d.Contributors()), 3 )
        self.failUnless( d.cooked_text.find('<p>') >= 0 )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(NewsItemTests),
        unittest.makeSuite(NewsItemPUTTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Default implementation of CMFCore.

$Id: __init__.py 73035 2007-03-07 16:41:03Z jens $
"""

from Products.CMFCore.utils import ToolInit
from Products.CMFCore.utils import ContentInit
from Products.CMFCore.utils import registerIcon

import DefaultWorkflow
import DiscussionTool
import Document
import factory
import Favorite
import File
import Image
import Link
import MembershipTool
import MetadataTool
import NewsItem
import Portal
import PropertiesTool
import RegistrationTool
import SkinnedFolder
import SyndicationTool
from permissions import AddPortalContent


# Make sure security is initialized
import DiscussionItem
import DublinCore
import utils

contentConstructors = ( Document.addDocument
                      , File.addFile
                      , Image.addImage
                      , Link.addLink
                      , Favorite.addFavorite
                      , NewsItem.addNewsItem
                      , SkinnedFolder.addSkinnedFolder
                      )

tools = ( DiscussionTool.DiscussionTool
        , MembershipTool.MembershipTool
        , RegistrationTool.RegistrationTool
        , PropertiesTool.PropertiesTool
        , MetadataTool.MetadataTool
        , SyndicationTool.SyndicationTool
        )

# Register the ID to interface data for those items that don't have their 
# own module
from Products.CMFCore.utils import registerToolInterface
from Products.GenericSetup.interfaces import ISetupTool
registerToolInterface('portal_setup', ISetupTool)
from Products.MailHost.interfaces import IMailHost
registerToolInterface('MailHost', IMailHost)

def initialize(context):

    ToolInit( 'CMF Default Tool'
            , tools=tools
            , icon='tool.gif'
            ).initialize( context )

    # BBB: register oldstyle constructors
    ContentInit( 'CMF Default Content'
               , content_types=()
               , permission=AddPortalContent
               , extra_constructors=contentConstructors
               ).initialize( context )

    context.registerClass( Portal.CMFSite
                         , constructors=(factory.addConfiguredSiteForm,
                                         factory.addConfiguredSite)
                         , icon='images/portal.gif'
                         )

    registerIcon( DefaultWorkflow.DefaultWorkflowDefinition
                , 'images/workflow.gif'
                , globals()
                )

    # make registerHelp work with 2 directories
    help = context.getProductHelp()
    lastRegistered = help.lastRegistered
    context.registerHelp(directory='help', clear=1)
    context.registerHelp(directory='interfaces', clear=1)
    if help.lastRegistered != lastRegistered:
        help.lastRegistered = None
        context.registerHelp(directory='help', clear=1)
        help.lastRegistered = None
        context.registerHelp(directory='interfaces', clear=0)
    context.registerHelpTitle('CMF Default Help')



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for Discussions.

$Id: test_Discussions.py 82817 2008-01-11 21:02:59Z yuppie $
"""

import unittest
import Testing

from zope.component import getSiteManager

from Products.CMFCore.CatalogTool import CatalogTool
from Products.CMFCore.interfaces import IDiscussionTool
from Products.CMFCore.testing import EventZCMLLayer
from Products.CMFCore.tests.base.dummy import DummyContent
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.dummy import DummyTool
from Products.CMFCore.tests.base.testcase import SecurityTest
from Products.CMFCore.tests.base.tidata import FTIDATA_DUMMY
from Products.CMFCore.tests.base.utils import has_path
from Products.CMFCore.TypesTool import FactoryTypeInformation as FTI
from Products.CMFCore.TypesTool import TypesTool
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DiscussionTool import DiscussionTool
from Products.CMFDefault.exceptions import DiscussionNotAllowed


class DiscussionItemTests(unittest.TestCase):

    def test_z2interfaces(self):
        from Interface.Verify import verifyClass
        from Products.CMFCore.interfaces.Contentish \
                import Contentish as IContentish
        from Products.CMFCore.interfaces.Discussions \
                import DiscussionResponse as IDiscussionResponse
        from Products.CMFCore.interfaces.DublinCore \
                import CatalogableDublinCore as ICatalogableDublinCore
        from Products.CMFCore.interfaces.DublinCore \
                import DublinCore as IDublinCore
        from Products.CMFCore.interfaces.DublinCore \
                import MutableDublinCore as IMutableDublinCore
        from Products.CMFCore.interfaces.Dynamic \
                import DynamicType as IDynamicType
        from Products.CMFDefault.DiscussionItem import DiscussionItem

        verifyClass(ICatalogableDublinCore, DiscussionItem)
        verifyClass(IContentish, DiscussionItem)
        verifyClass(IDiscussionResponse, DiscussionItem)
        verifyClass(IDublinCore, DiscussionItem)
        verifyClass(IDynamicType, DiscussionItem)
        verifyClass(IMutableDublinCore, DiscussionItem)

    def test_z3interfaces(self):
        from zope.interface.verify import verifyClass
        from Products.CMFCore.interfaces import ICatalogableDublinCore
        from Products.CMFCore.interfaces import IContentish
        from Products.CMFCore.interfaces import IDiscussionResponse
        from Products.CMFCore.interfaces import IDublinCore
        from Products.CMFCore.interfaces import IDynamicType
        from Products.CMFCore.interfaces import IMutableDublinCore
        from Products.CMFDefault.DiscussionItem import DiscussionItem

        verifyClass(ICatalogableDublinCore, DiscussionItem)
        verifyClass(IContentish, DiscussionItem)
        verifyClass(IDiscussionResponse, DiscussionItem)
        verifyClass(IDublinCore, DiscussionItem)
        verifyClass(IDynamicType, DiscussionItem)
        verifyClass(IMutableDublinCore, DiscussionItem)


class DiscussionItemContainerTests(unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFDefault.DiscussionItem import DiscussionItemContainer

        return DiscussionItemContainer

    def test_z2interfaces(self):
        from Interface.Verify import verifyClass
        from Products.CMFCore.interfaces.Discussions \
                import Discussable as IDiscussable

        verifyClass(IDiscussable, self._getTargetClass())

    def test_z3interfaces(self):
        from zope.interface.verify import verifyClass
        from Products.CMFCore.interfaces import ICallableOpaqueItemEvents
        from Products.CMFCore.interfaces import IDiscussable

        verifyClass(ICallableOpaqueItemEvents, self._getTargetClass())
        verifyClass(IDiscussable, self._getTargetClass())


class DiscussionTests(SecurityTest):

    layer = EventZCMLLayer

    def setUp(self):
        SecurityTest.setUp(self)
        self.site = DummySite('site').__of__(self.root)
        sm = getSiteManager()
        self.site._setObject( 'portal_discussion', DiscussionTool() )
        sm.registerUtility(self.site.portal_discussion, IDiscussionTool)
        self.site._setObject( 'portal_membership', DummyTool() )
        self.site._setObject( 'portal_types', TypesTool() )

    def _makeDummyContent(self, id, *args, **kw):
        return self.site._setObject( id, DummyContent(id, *args, **kw) )

    def test_policy( self ):
        dtool = self.site.portal_discussion
        ttool = self.site.portal_types
        test = self._makeDummyContent('test')
        self.assertRaises(DiscussionNotAllowed, dtool.getDiscussionFor, test)
        assert getattr( test, 'talkback', None ) is None

        test.allow_discussion = 1
        assert dtool.getDiscussionFor(test)
        assert test.talkback

        del test.talkback
        del test.allow_discussion
        fti = FTIDATA_DUMMY[0].copy()
        ttool._setObject( 'Dummy Content', FTI(**fti) )
        self.assertRaises(DiscussionNotAllowed, dtool.getDiscussionFor, test)
        assert getattr( test, 'talkback', None ) is None

        ti = getattr(ttool, 'Dummy Content')
        ti.allow_discussion = 1
        assert dtool.getDiscussionFor(test)
        assert test.talkback

        del test.talkback
        ti.allow_discussion = 0
        self.assertRaises(DiscussionNotAllowed, dtool.getDiscussionFor, test)
        assert getattr( test, 'talkback', None ) is None

        test.allow_discussion = 1
        assert dtool.getDiscussionFor(test)
        assert test.talkback

    def test_nestedReplies( self ):
        dtool = self.site.portal_discussion
        test = self._makeDummyContent('test')
        test.allow_discussion = 1
        talkback = dtool.getDiscussionFor(test)
        assert talkback._getDiscussable() == test
        assert talkback._getDiscussable( outer=1 ) == test
        assert not talkback.hasReplies( test )
        assert len( talkback.getReplies() ) == 0

        reply_id = talkback.createReply( title='test', text='blah' )
        assert talkback.hasReplies( test )
        assert len( talkback.getReplies() ) == 1
        assert talkback.getReply( reply_id )

        reply1 = talkback.getReplies()[0]
        items = talkback._container.items()
        self.assertEqual( reply1.getId(), items[0][0] )
        self.assertEqual( reply1.inReplyTo(), test )
        self.assertEqual( reply1.listCreators(), ('dummy',) )

        parents = reply1.parentsInThread()
        assert len( parents ) == 1
        assert test in parents

        talkback1 = dtool.getDiscussionFor(reply1)
        assert talkback == talkback1
        assert len( talkback1.getReplies() ) == 0
        assert len( talkback.getReplies() ) == 1

        talkback1.createReply( title='test2'
                             , text='blah2'
                             )
        assert len( talkback._container ) == 2
        assert talkback1.hasReplies( reply1 )
        assert len( talkback1.getReplies() ) == 1
        assert len( talkback.getReplies() ) == 1

        reply2 = talkback1.getReplies()[0]
        assert reply2.inReplyTo() == reply1

        parents = reply2.parentsInThread()
        assert len( parents ) == 2
        assert parents[ 0 ] == test
        assert parents[ 1 ] == reply1

        parents = reply2.parentsInThread( 1 )
        assert len( parents ) == 1
        assert parents[ 0 ] == reply1

    def test_itemCataloguing( self ):
        ctool = self.site._setObject( 'portal_catalog', CatalogTool() )
        ctool.addColumn('in_reply_to')
        dtool = self.site.portal_discussion
        catalog = ctool._catalog
        test = self._makeDummyContent('test', catalog=1)
        test.allow_discussion = 1

        self.assertEqual( len(ctool), 1 )
        self.failUnless( has_path( catalog, test.getPhysicalPath() ) )
        talkback = dtool.getDiscussionFor(test)
        self.assertEqual( talkback.getPhysicalPath(),
                          ('', 'bar', 'site', 'test', 'talkback') )
        talkback.createReply( title='test'
                            , text='blah'
                            )
        self.assertEqual( len(ctool), 2 )
        for reply in talkback.getReplies():
            self.failUnless( has_path( catalog, reply.getPhysicalPath() ) )
            self.failUnless( has_path( catalog,
                              '/bar/site/test/talkback/%s' % reply.getId() ) )

        reply1 = talkback.getReplies()[0]
        path1 = '/'.join(reply1.getPhysicalPath())
        self.assertEqual( ctool.getMetadataForUID(path1),
                          {'in_reply_to': None} )

        talkback1 = dtool.getDiscussionFor(reply1)
        talkback1.createReply( title='test2'
                             , text='blah2'
                             )
        for reply in talkback.getReplies():
            self.failUnless( has_path( catalog, reply.getPhysicalPath() ) )
            self.failUnless( has_path( catalog,
                              '/bar/site/test/talkback/%s' % reply.getId() ) )
        for reply in talkback1.getReplies():
            self.failUnless( has_path( catalog, reply.getPhysicalPath() ) )
            self.failUnless( has_path( catalog,
                              '/bar/site/test/talkback/%s' % reply.getId() ) )

        reply2 = talkback1.getReplies()[0]
        path2 = '/'.join(reply2.getPhysicalPath())
        self.assertEqual( ctool.getMetadataForUID(path2),
                          {'in_reply_to': reply1.getId()} )

    def test_itemWorkflowNotification(self):
        from Products.CMFDefault.DiscussionItem import DiscussionItem

        dtool = self.site.portal_discussion
        test = self._makeDummyContent('test')
        test.allow_discussion = 1
        talkback = dtool.getDiscussionFor(test)

        # Monkey patch into the class to test, urgh.
        def notifyWorkflowCreated(self):
            self.test_wf_notified = 1
            DiscussionItem.inheritedAttribute('notifyWorkflowCreated')(self)
        old_method = getattr(DiscussionItem, 'notifyWorkflowCreated', None)
        DiscussionItem.notifyWorkflowCreated = notifyWorkflowCreated
        DiscussionItem.test_wf_notified = 0

        try:
            reply_id = talkback.createReply(title='test', text='blah')
            reply = talkback.getReplies()[0]
            self.assertEqual(reply.test_wf_notified, 1)
        finally:
            delattr(DiscussionItem, 'test_wf_notified')
            if old_method is None:
                delattr(DiscussionItem, 'notifyWorkflowCreated')
            else:
                DiscussionItem.notifyWorkflowCreated = old_method

    def test_deletePropagation( self ):
        ctool = self.site._setObject( 'portal_catalog', CatalogTool() )
        dtool = self.site.portal_discussion
        test = self._makeDummyContent('test', catalog=1)
        test.allow_discussion = 1

        talkback = dtool.getDiscussionFor(test)
        talkback.createReply( title='test'
                            , text='blah'
                            )
        self.assertEqual( len(ctool), 2 )
        self.site._delObject('test')
        self.assertEqual( len(ctool), 0 )

    def test_deleteReplies(self):
        dtool = self.site.portal_discussion
        ctool = self.site._setObject( 'portal_catalog', CatalogTool() )
        test = self._makeDummyContent('test')
        test.allow_discussion = 1

        # Create a structure 6 levels deep for testing
        talkback = dtool.getDiscussionFor(test)
        id1 = talkback.createReply(title='test1', text='blah')
        reply1 = talkback.getReply(id1)
        talkback1 = dtool.getDiscussionFor(reply1)
        id2 = talkback1.createReply(title='test2', text='blah')
        reply2 = talkback1.getReply(id2)
        talkback2 = dtool.getDiscussionFor(reply2)
        id3 = talkback2.createReply(title='test3', text='blah')
        reply3 = talkback2.getReply(id3)
        talkback3 = dtool.getDiscussionFor(reply3)
        id4 = talkback3.createReply(title='test4', text='blah')
        reply4 = talkback3.getReply(id4)
        talkback4 = dtool.getDiscussionFor(reply4)
        id5 = talkback4.createReply(title='test5', text='blah')
        reply5 = talkback4.getReply(id5)
        talkback5 = dtool.getDiscussionFor(reply5)
        id6 = talkback5.createReply(title='test6', text='blah')
        reply6 = talkback5.getReply(id6)
        talkback6 = dtool.getDiscussionFor(reply6)

        self.assertEqual(len(talkback.getReplies()), 1)
        self.assertEqual(len(talkback1.getReplies()), 1)
        self.assertEqual(len(talkback2.getReplies()), 1)
        self.assertEqual(len(talkback3.getReplies()), 1)
        self.assertEqual(len(talkback4.getReplies()), 1)
        self.assertEqual(len(talkback5.getReplies()), 1)
        self.assertEqual(len(talkback6.getReplies()), 0)
        self.assertEqual(len(ctool), 7)

        talkback3.deleteReply(id4)
        self.assertEqual(len(talkback.getReplies()), 1)
        self.assertEqual(len(talkback1.getReplies()), 1)
        self.assertEqual(len(talkback2.getReplies()), 1)
        self.assertEqual(len(talkback3.getReplies()), 0)
        self.assertEqual(len(ctool), 4)

        talkback.deleteReply(id1)
        self.assertEqual(len(talkback.getReplies()), 0)
        self.assertEqual(len(ctool), 1)

    def test_newTalkbackIsWrapped(self):
        test = self._makeDummyContent('test')
        test.allow_discussion = 1
        dtool = self.site.portal_discussion
        talkback = dtool.getDiscussionFor(test)
        self.failUnless(hasattr(talkback, 'aq_base'))
        # Acquire a portal tool
        self.failUnless(getToolByName(talkback, 'portal_discussion'))

    def test_existingTalkbackIsWrapped(self):
        test = self._makeDummyContent('test')
        test.allow_discussion = 1
        dtool = self.site.portal_discussion
        talkback = dtool.getDiscussionFor(test)
        talkback = dtool.getDiscussionFor(test)
        self.failUnless(hasattr(talkback, 'aq_base'))
        # Acquire a portal tool
        self.failUnless(getToolByName(talkback, 'portal_discussion'))


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(DiscussionItemTests),
        unittest.makeSuite(DiscussionItemContainerTests),
        unittest.makeSuite(DiscussionTests),
        ))

if __name__ == '__main__':
    from Products.CMFCore.testing import run
    run(test_suite())



##parameters=
##
from Products.CMFDefault.utils import decode

options = {}
options['title'] = context.Title()
options['description'] = context.Description()
options['text'] = context.CookedBody()

return context.document_view_template(**decode(options, script))



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" CMFDefault setup handlers.

$Id: setuphandlers.py 68358 2006-05-29 19:42:51Z yuppie $
"""

from exceptions import BadRequest


def importVarious(context):
    """ Import various settings.

    This provisional handler will be removed again as soon as full handlers
    are implemented for these steps.
    """
    site = context.getSite()

    try:
        site.manage_addPortalFolder('Members')
    except BadRequest:
        return 'Various settings: Nothing to import.'
    site.Members.manage_addProduct['OFSP'].manage_addDTMLMethod('index_html',
                                        'Member list', '<dtml-return roster>')
    site.acl_users.encrypt_passwords = False

    return 'Various settings from PortalGenerator imported.'



##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for DublinCore module.

$Id: test_DublinCore.py 77186 2007-06-28 19:06:19Z yuppie $
"""

import unittest
import Testing

from AccessControl.SecurityManagement import newSecurityManager
from Acquisition import Implicit
from DateTime.DateTime import DateTime
from zope.testing.cleanup import cleanUp

from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.dummy import DummyUserFolder
from Products.CMFCore.tests.base.testcase import SecurityTest
from Products.CMFDefault.MembershipTool import MembershipTool


def _DateIndexConvert(value):
    # Duplicate date conversion done by DateIndex._convert
    t_tup = value.toZone('UTC').parts()
    yr = t_tup[0]
    mo = t_tup[1]
    dy = t_tup[2]
    hr = t_tup[3]
    mn = t_tup[4]
    t_val = ((((yr * 12 + mo) * 31 + dy) * 24 + hr) * 60 + mn)

    if isinstance(t_val, long):
        # t_val must be IntType, not LongType
        raise OverflowError("Date too big: %s" % `value`)

    return t_val


class DummyMetadataTool(Implicit):

    def __init__(self, publisher):
        self._publisher = publisher

    def getPublisher(self):
        return self._publisher


class DublinCoreTests(SecurityTest):

    def _makeDummyContent(self, id, *args, **kw):
        from Products.CMFCore.PortalContent import PortalContent
        from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

        class DummyContent(PortalContent, DefaultDublinCoreImpl):
            pass

        return DummyContent(id, *args, **kw)

    def tearDown(self):
        cleanUp()
        SecurityTest.tearDown(self)

    def test_z2interfaces(self):
        from Interface.Verify import verifyClass
        from Products.CMFCore.interfaces.DublinCore \
                import CatalogableDublinCore as ICatalogableDublinCore
        from Products.CMFCore.interfaces.DublinCore \
                import DublinCore as IDublinCore
        from Products.CMFCore.interfaces.DublinCore \
                import MutableDublinCore as IMutableDublinCore
        from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

        verifyClass(ICatalogableDublinCore, DefaultDublinCoreImpl)
        verifyClass(IDublinCore, DefaultDublinCoreImpl)
        verifyClass(IMutableDublinCore, DefaultDublinCoreImpl)

    def test_z3interfaces(self):
        from zope.interface.verify import verifyClass
        from Products.CMFCore.interfaces import ICatalogableDublinCore
        from Products.CMFCore.interfaces import IDublinCore
        from Products.CMFCore.interfaces import IMutableDublinCore
        from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

        verifyClass(ICatalogableDublinCore, DefaultDublinCoreImpl)
        verifyClass(IDublinCore, DefaultDublinCoreImpl)
        verifyClass(IMutableDublinCore, DefaultDublinCoreImpl)

    def test_notifyModified(self):
        site = DummySite('site').__of__(self.root)
        acl_users = site._setObject( 'acl_users', DummyUserFolder() )
        site._setObject( 'portal_membership', MembershipTool() )
        newSecurityManager(None, acl_users.user_foo)
        item = self._makeDummyContent('item').__of__(site)
        self.assertEqual( item.listCreators(), () )
        item.setModificationDate(0)
        initial_date = item.ModificationDate()

        item.notifyModified()
        self.assertEqual( item.listCreators(), ('user_foo',) )
        self.assertNotEqual( item.ModificationDate(), initial_date )

    def test_creators_methods(self):
        site = DummySite('site').__of__(self.root)
        acl_users = site._setObject( 'acl_users', DummyUserFolder() )
        site._setObject( 'portal_membership', MembershipTool() )
        newSecurityManager(None, acl_users.user_foo)
        item = self._makeDummyContent('item').__of__(site)
        self.assertEqual( item.listCreators(), () )

        item.addCreator()
        self.assertEqual( item.listCreators(), ('user_foo',) )
        newSecurityManager(None, acl_users.user_bar)
        item.addCreator()
        self.assertEqual( item.listCreators(), ('user_foo', 'user_bar') )
        item.addCreator()
        self.assertEqual( item.listCreators(), ('user_foo', 'user_bar') )
        item.addCreator('user_baz')
        self.assertEqual( item.listCreators(),
                          ('user_foo', 'user_bar', 'user_baz') )
        item.setCreators('user_bar')
        self.assertEqual( item.listCreators(), ('user_bar',) )
        item.setCreators( ('user_baz',) )
        self.assertEqual( item.listCreators(), ('user_baz',) )

    def test_creators_upgrade(self):
        site = DummySite('site').__of__(self.root)
        acl_users = site._setObject( 'acl_users', DummyUserFolder() )
        site._setObject( 'portal_membership', MembershipTool() )
        newSecurityManager(None, acl_users.user_foo)
        item = self._makeDummyContent('item').__of__(site)
        item.manage_fixupOwnershipAfterAdd()
        # fake an old object < CMF 1.5 without creators
        delattr(item, 'creators')
        self.assertEqual(item.Creator(), 'user_foo')
        newSecurityManager(None, acl_users.user_bar)
        item.addCreator()
        self.assertEqual(item.Creator(), 'user_foo')
        self.assertEqual(item.listCreators(), ('user_foo', 'user_bar'))
        # or if added directly
        delattr(item, 'creators')
        item.addCreator()
        self.assertEqual(item.Creator(), 'user_foo')
        self.assertEqual(item.listCreators(), ('user_foo', 'user_bar'))

    def test_ceiling_parsable(self):
        # Test that a None ceiling date will be parsable by a DateIndex
        site = DummySite('site').__of__(self.root)
        item = self._makeDummyContent('item').__of__(site)
        self.assertEqual(item.expiration_date, None)
        self.assert_(_DateIndexConvert(item.expires()))

    def test_publisher_no_metadata_tool(self):
        site = DummySite('site').__of__(self.root)
        item = self._makeDummyContent('item').__of__(site)
        self.assertEqual(item.Publisher(), 'No publisher')

    def test_publisher_with_metadata_tool(self):
        PUBLISHER = 'Some Publisher'
        site = DummySite('site').__of__(self.root)
        site.portal_metadata = DummyMetadataTool(publisher=PUBLISHER)
        item = self._makeDummyContent('item').__of__(site)
        self.assertEqual(item.Publisher(), PUBLISHER)

    def test_timezone_metadata(self):
        # http://www.zope.org/Collectors/CMF/325
        # If an item's timestamp(s) are stored in another timezone,
        # e.g. 4 hours further away from UTC, the DC date methods
        # should still return it in the local timezone so that all
        # user-visible dates can be compared to each other by eye.
        site = DummySite('site').__of__(self.root)
        item = self._makeDummyContent('item').__of__(site)
        dates_and_methods = (
            ('modification_date', 'ModificationDate'),
            ('effective_date', 'EffectiveDate'),
            ('effective_date', 'Date'),
            ('expiration_date', 'ExpirationDate'),
            ('creation_date', 'CreationDate'))
        offset = 4  # arbitrary, any value should work.
        for datename, dc_methodname in dates_and_methods:
            orig = getattr(item, datename)
            # Some default to None, fix that.
            if orig is None:
                orig = DateTime()
                setattr(item, datename, orig)
            orig_DC = getattr(item, dc_methodname)()
            # Change the timezone of the date.
            local_offset = orig.tzoffset() % (3600*24)
            other_offset = (local_offset + offset) % 24
            otherzone = 'GMT+%d' % other_offset
            setattr(item, datename, orig.toZone(otherzone))
            # Finally, verify that display has not changed.
            new_DC = getattr(item, dc_methodname)()
            self.assertEqual(orig_DC, new_DC)

    def test_Date_with_explicit_timezone(self):
        item = self._makeDummyContent('item')
        item.effective_date = DateTime('2007-01-01T12:00:00Z')
        self.assertEqual(item.Date('US/Eastern'),
                         '2007-01-01 07:00:00')

    def test_CreationDate_with_explicit_timezone(self):
        item = self._makeDummyContent('item')
        item.creation_date = DateTime('2007-01-01T12:00:00Z')
        self.assertEqual(item.CreationDate('US/Eastern'),
                         '2007-01-01 07:00:00')

    def test_ModificationDate_with_explicit_timezone(self):
        item = self._makeDummyContent('item')
        item.modification_date = DateTime('2007-01-01T12:00:00Z')
        self.assertEqual(item.ModificationDate('US/Eastern'),
                         '2007-01-01 07:00:00')

    def test_EffectiveDate_with_explicit_timezone(self):
        item = self._makeDummyContent('item')
        item.effective_date = DateTime('2007-01-01T12:00:00Z')
        self.assertEqual(item.EffectiveDate('US/Eastern'),
                         '2007-01-01 07:00:00')

    def test_ExpirationDate_with_explicit_timezone(self):
        item = self._makeDummyContent('item')
        item.expiration_date = DateTime('2007-01-01T12:00:00Z')
        self.assertEqual(item.ExpirationDate('US/Eastern'),
                         '2007-01-01 07:00:00')

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(DublinCoreTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##parameters=ids=(), **kw
##
from Products.CMFDefault.utils import Message as _

if ids:
    return context.setStatus(True)
else:
    return context.setStatus(False, _(u'Please select one or more items '
                                      u'first.'))



from Products.CMFCore.TypesTool import FactoryTypeInformation
from Products.CMFDefault import DiscussionItem

def update_discussion(self):
    """
     1. Install (if it isn't there already) a type information
        object for DiscussionItems, so that they can get actions,
        etc.  Erase the "(default)" workflow bound to it, to prevent
        showing the "Retract" options, etc.

     2. Update all DiscussionItems to use the new marking for
        'in_reply_to':

          - Items which are replies to the containing content object
            have None as their 'in_reply_to';

          - Items which are replies to sibling items have the sibling's
            ID as their 'in_reply_to'.

        The representation we are converting from was:

          - Items which are replies to the containing content object
            have the portal-relative pathstring of the content object
            as their 'in_reply_to';

          - Items which are replies to sibling items have the absolute
            path of the sibling as their 'in_reply_to'.
    """

    log = []
    a = log.append
    types_tool = self.portal_types
    if not getattr( types_tool, 'Discussion Item', None ):

        fti = FactoryTypeInformation(
                                **DiscussionItem.factory_type_information[0] )
        types_tool._setObject( 'Discussion Item', fti )
        a( 'Added type object for DiscussionItem' )

        workflow_tool = self.portal_workflow
        workflow_tool.setChainForPortalTypes( ( 'Discussion Item', ), () )
        a( 'Erased workflow for DiscussionItem' )

    items = self.portal_catalog.searchResults( meta_type='Discussion Item' )
    a( 'DiscussionItems updated:' )

    for item in items:

        object = item.getObject()
        talkback = object.aq_parent
        path = item.getPath()
        in_reply_to = object.in_reply_to

        if in_reply_to is None: # we've been here already
            continue

        irt_elements = in_reply_to.split('/')

        if len( irt_elements ) == 1:
            if talkback._container.get( irt_elements[0] ):
                # we've been here already
                continue

        if irt_elements[0] == '': # absolute, so we are IRT a sibling
            sibling_id = irt_elements[ -1 ]
            if talkback._container.get( sibling_id, None ):
                in_reply_to = sibling_id
            else:
                in_reply_to = None
        else:
            in_reply_to = None

        object.in_reply_to = in_reply_to
        assert object.inReplyTo() # sanity check
        object.reindexObject()

        a( path )

    return '\n'.join(log)




##parameters=
##title=Delete reply
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import Message as _

dtool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IDiscussionTool')

parent = context.inReplyTo()
talkback = dtool.getDiscussionFor(parent)
talkback.deleteReply( context.getId() )

context.setStatus(True, _(u'Reply deleted.'))
context.setRedirect(parent, 'object/view')



from Acquisition import aq_base, aq_inner, aq_parent
from Globals import PersistentMapping
import sys


#
# Routines generally useful for migration purposes.
#


class Converter:
    def allowDescendChildren(self): raise 'Not implemented'
    def convert(self, ob): raise 'Not implemented'
    def showDuplicationError(self): raise 'Not implemented'


class Migrator:

    def __init__(self, conversions, skip):
        self.conversions = conversions
        self.skip = skip
        self.visited_folders = []
        self.warnings = []
        self.copied = []
        self.skipped = []

    def migrateObjectManager(self, src_folder, dst_folder, place=()):
        self.visited_folders.append( '/'.join(place) )
        for id, s_ob in src_folder.objectItems():
            d_ob = getattr(dst_folder, id, None)
            to_store = self.migrateObject(id, s_ob, d_ob, dst_folder,
                                          place + (id,))
            if to_store is not None:
                owner = getattr(to_store, '_owner', None)
                if hasattr(dst_folder, '_setObject'):
                    dst_folder._setObject(id, to_store)
                else:
                    setattr(dst_folder, id, to_store)
                if owner is not None:
                    # Retain ownership.
                    to_store._owner = owner

    def migrateDiscussionContainer(self, src_folder, dst_folder, place=()):
        self.visited_folders.append( '/'.join(place) )
        dst_container = getattr(dst_folder, '_container', None)
        if dst_container is None:
            dst_container = dst_folder._container = PersistentMapping()
        for id, s_ob in src_folder._container.items():
            d_ob = dst_container.get(id)
            to_store = self.migrateObject(id, s_ob, d_ob, dst_folder,
                                          place + (id,))
            if to_store is not None:
                dst_container[id] = aq_base(to_store)

    def migratePossibleContainer(self, s_ob, d_ob, place):
        base_ob = aq_base(s_ob)
        if hasattr(base_ob, 'objectItems'):
            self.migrateObjectManager(s_ob, d_ob, place)
        elif hasattr(base_ob, '_container'):
            self.migrateDiscussionContainer(s_ob, d_ob, place)

    def migrateObject(self, id, s_ob, d_ob, dst_folder, place):
        # Doesn't store changes, only returns the
        # object to store.
        conversions = self.conversions
        klass = s_ob.__class__
        descend_ok = 1
        base_ob = aq_base(s_ob)
        to_store = None
        pathname = '/'.join(place)
        if self.skip.has_key(id):
            # Don't migrate objects by this name, but we can still
            # migrate subobjects.
            descend_ok = self.skip[id]
            if descend_ok and d_ob is None:
                descend_ok = 0
            self.skipped.append(pathname +
                               (descend_ok and ' (descended)' or ' '))
        elif d_ob is not None:
            # The dest already has something with this ID.
            descend_ok = 1
            show_message = 1
            converter = conversions.get(klass, None)
            if converter is not None:
                descend_ok = converter.allowDescendChildren()
                show_message = converter.showDuplicationError()
            if show_message:
                self.warnings.append('Already existed: %s' % pathname)
        elif conversions.has_key(klass):
            # Invoke the appropriate converter.
            converter = conversions[klass]
            to_store = converter.convert(s_ob)
            self.copied.append(pathname)
        elif hasattr(base_ob, '_getCopy'):
            # Make a direct copy.
            to_store = s_ob._getCopy(dst_folder)
            self.warnings.append('Copied %s directly.' % pathname)
            descend_ok = 0
        else:
            # No way to copy.
            descend_ok = 0
            self.warnings.append('Could not copy %s' % pathname)
        if descend_ok:
            if to_store is not None:
                d_ob = to_store
            if d_ob is not None:
                try: d_ob._p_jar = dst_folder._p_jar
                except: pass
                self.migratePossibleContainer(s_ob, d_ob, place)
        return to_store


class SimpleClassConverter (Converter):
    def __init__(self, to_class, descend, show_dup=1):
        self._klass = to_class
        self._descend = descend
        self._show_dup = show_dup

    def allowDescendChildren(self):
        return self._descend

    def showDuplicationError(self):
        return self._show_dup

    def convert(self, ob):
        # Creates a copy of ob without its children.
        ob = aq_base(ob)
        k = self._klass
        if hasattr(k, '__basicnew__'):
            newob = k.__basicnew__()
        else:
            newob = new.instance(k, {})
        id = ob.id
        if callable(id):
            id = id()
        try: newob._setId(id)
        except AttributeError: newob.id = id
        newob.__dict__.update(ob.__dict__)
        if hasattr(newob, '_objects'):
            # Clear the children.
            for info in newob._objects:
                del newob.__dict__[info['id']]
            newob._objects = ()
        if hasattr(newob, '_container'):
            # Clear the children.
            newob._container = PersistentMapping()
        return newob

TupleType = type(())

def setupDirectConversion(old_prod, new_prod, modname, classname,
                          conversions, descend=1, show_dup=1):
    try:
        old_module = sys.modules['Products.' + old_prod + '.' + modname]
        new_module = sys.modules['Products.' + new_prod + '.' + modname]
        old_class = getattr(old_module, classname)
        new_class = getattr(new_module, classname)
        conversions[old_class] = SimpleClassConverter(new_class, descend,
                                                      show_dup)
    except:
        print 'Failed to set up conversion', old_prod, new_prod, modname, classname
        import traceback
        traceback.print_exc()


def setupDirectConversions(old_prod, new_prod, modnames, conversions):
    for info in modnames:
        if type(info) is TupleType:
            modname, classname = info
        else:
            modname = classname = info
        setupDirectConversion(old_prod, new_prod, modname, classname,
                              conversions)


def _cleanupOwnership(ob, res, cleanup_children):
    '''
    If the user name of the owner of the referenced object
    is not found in its current user database but is found
    in the local user database, this function changes the
    ownership of the object to the local database.
    '''
    try: changed = ob._p_changed
    except: changed = 0

    owner = getattr(ob, '_owner', None)
    if owner:
        udb, uid = owner
        #res.append('Owner of %s is %s!%s' % (
        #    '/'.join( ob.getPhysicalPath() ), '/'.join(udb), uid,))
        root = ob.getPhysicalRoot()
        try:
            db = root.unrestrictedTraverse(udb, None)
            user = db.getUserById(uid)
            if hasattr(ob, 'aq_inContextOf'):
                ucontext = aq_parent(aq_inner(db))
                if not ob.aq_inContextOf(ucontext):
                    # Not in the right context.
                    user = None
        except:
            user = None
        if user is None:
            # Try to change to a local database.
            p = ob
            old_udb = udb
            udb = None
            while p is not None:
                if hasattr(p, 'acl_users'):
                    acl_users = p.acl_users
                    try:
                        user = acl_users.getUserById(uid)
                    except:
                        user = None
                    if user is not None:
                        # Found the right database.
                        udb = acl_users.getPhysicalPath()[1:]
                        break
                p = aq_parent(aq_inner(p))
            if udb is not None:
                ob._owner = udb, uid
                res.append('Changed ownership of %s from %s!%s to %s!%s' %
                           ('/'.join( ob.getPhysicalPath() ),
                            '/'.join(old_udb), uid,
                            '/'.join(udb), uid,))
            else:
                res.append('Could not fix the ownership of %s, '
                           'which is set to %s!%s' %
                           ('/'.join( ob.getPhysicalPath() ),
                            '/'.join(old_udb), uid,))

    if cleanup_children:
        if hasattr(ob, 'objectValues'):
            for subob in ob.objectValues():
                _cleanupOwnership(subob, res, 1)

    # Deactivate object if possible.
    if changed is None: ob._p_deactivate()

    return res

def _copyUsers(src_folder, dst_folder):
    source = src_folder.acl_users
    target = dst_folder.acl_users
    for user in source.getUsers():
        target._addUser(name=user.name, password=user.__, confirm=user.__,
                        roles=user.roles, domains=user.domains, REQUEST=None)

#
# PTK to CMF Migration script.
#


def migrate(self, src_path='', dest_path='', copy_users=0, ownership_only=0):
    if not src_path or not dest_path:
        return '''
        <html><body><form action="%s" method="POST">
        <h2>Migrate PTK content to CMF site</h2>
        <p>Path (not including server URL) to PTK instance (source):
        <input type="text" name="src_path"></p>
        <p>Path (not including server URL) to CMF site (destination):
        <input type="text" name="dest_path"></p>
        <p>Copy users:
        <input type="checkbox" name="copy_users" value="1"></p>
        <input type="submit" name="submit" value="Migrate">
        <input type="submit" name="ownership_only"
        value="Just clean up ownership">
        </form></body></html>
        ''' % self.REQUEST['URL']
    root = self.getPhysicalRoot()
    dst_folder = root.restrictedTraverse(dest_path)
    if not ownership_only:
        src_folder = root.restrictedTraverse(src_path)
        if copy_users:
            _copyUsers(src_folder, dst_folder)
        m = Migrator(ptk2cmf_conversions, ptk2cmf_skip)
        m.migrateObjectManager(src_folder, dst_folder)
    ownership_res = []
    _cleanupOwnership(dst_folder, ownership_res, 1)
    return '''
        <html><body><p>Finished migration.</p>
        <p>Warnings (if any):<ul><li>%s</li></ul></p>
        <p>Visited folders:<ul><li>%s</li></ul></p>
        <p>Skipped:<ul><li>%s</li></ul></p>
        <p>Converted content:</p><pre>%s</pre>
        <p>Fixed up ownership:</p><pre>%s</pre>
        </body></html>
        ''' % ('</li>\n<li>'.join(m.warnings),
               '</li>\n<li>'.join(m.visited_folders),
               '</li>\n<li>'.join(m.skipped),
               '\n'.join(m.copied),
               '\n'.join(ownership_res),
               )

migrate_ptk = migrate

#
# PTK to CMF Conversion definitions.
#


ptk2cmf_conversions = {}

ptk2cmf_skip = {
    'portal_actions':0,
    'portal_catalog':0,
    'portal_discussion':0,
    'portal_memberdata':0,
    'portal_membership':0,
    'portal_properties':0,
    'portal_registration':0,
    'portal_skins':1,
    'portal_types':0,
    'portal_undo':0,
    'portal_url':0,
    'portal_workflow':0,
    'MailHost':0,
    'cookie_authentication':0,
    }

demo_conversions = (
    'Document',
    'NewsItem',
    'Image',
    'File',
    'Link',
    'Favorite',
    'DiscussionItem',
    ('DiscussionItem', 'DiscussionItemContainer'),
    )


BEFORE_CONTENT_MOVE = 0

if BEFORE_CONTENT_MOVE:
    content_product = 'PTKBase'
else:
    content_product = 'PTKDemo'


setupDirectConversions(content_product, 'CMFDefault', demo_conversions,
                       ptk2cmf_conversions)

setupDirectConversion('PTKBase', 'CMFCore', 'DirectoryView', 'DirectoryView',
                       ptk2cmf_conversions, 0, 0)

setupDirectConversion('PTKBase', 'CMFCore', 'PortalFolder', 'PortalFolder',
                       ptk2cmf_conversions, 1)

from OFS.Folder import Folder
from Products.CMFCore.PortalFolder import PortalFolder
ptk2cmf_conversions[Folder] = SimpleClassConverter(PortalFolder, 1)



##parameters=transaction_info
##title=Undo transactions
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import Message as _

utool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IUndoTool')

utool.undo(context, transaction_info)

context.setStatus(True, _(u'Transaction(s) undone.'))
context.setRedirect(context, 'object/folderContents')



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Basic portal discussion access tool.

$Id: DiscussionTool.py 81923 2007-11-18 22:51:13Z jens $
"""

from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from Globals import DTMLFile
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from zope.interface import implements

from Products.CMFCore.interfaces import IDiscussionResponse
from Products.CMFCore.interfaces import IDiscussionTool
from Products.CMFCore.interfaces.Discussions \
        import DiscussionResponse as z2IDiscussionResponse
from Products.CMFCore.interfaces.portal_discussion \
        import portal_discussion as z2IDiscussionTool
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import registerToolInterface
from Products.CMFCore.utils import UniqueObject

from DiscussionItem import DiscussionItemContainer
from exceptions import AccessControl_Unauthorized
from exceptions import DiscussionNotAllowed
from permissions import ManagePortal
from permissions import ModifyPortalContent
from utils import _dtmldir

_marker = []


class DiscussionTool(UniqueObject, SimpleItem):

    """ Links content to discussions.
    """

    implements(IDiscussionTool)
    __implements__ = (z2IDiscussionTool, )

    id = 'portal_discussion'
    meta_type = 'Default Discussion Tool'

    security = ClassSecurityInfo()

    manage_options = ( ({'label': 'Overview',
                         'action': 'manage_overview'},)
                     + SimpleItem.manage_options
                     )

    #
    #   ZMI methods
    #
    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = DTMLFile( 'explainDiscussionTool', _dtmldir )

    #
    #   'portal_discussion' interface methods
    #

    security.declarePublic( 'overrideDiscussionFor' )
    def overrideDiscussionFor(self, content, allowDiscussion):
        """ Override discussability for the given object or clear the setting.
        """
        if not _checkPermission(ModifyPortalContent, content):
            raise AccessControl_Unauthorized

        if allowDiscussion is None or allowDiscussion == 'None':
            disc_flag = getattr(aq_base(content), 'allow_discussion', _marker)
            if disc_flag is not _marker:
                try:
                    del content.allow_discussion
                except AttributeError:
                    # https://bugs.launchpad.net/zope-cmf/+bug/162532
                    pass
        else:
            content.allow_discussion = bool(allowDiscussion)

    security.declarePublic( 'getDiscussionFor' )
    def getDiscussionFor(self, content):
        """ Get DiscussionItemContainer for content, create it if necessary.
        """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed

        if not IDiscussionResponse.providedBy(content) and \
                not z2IDiscussionResponse.isImplementedBy(content) and \
                getattr( aq_base(content), 'talkback', None ) is None:
            # Discussion Items use the DiscussionItemContainer object of the
            # related content item, so only create one for other content items
            self._createDiscussionFor(content)

        return content.talkback # Return wrapped talkback

    security.declarePublic( 'isDiscussionAllowedFor' )
    def isDiscussionAllowedFor( self, content ):
        """ Get boolean indicating whether discussion is allowed for content.
        """
        if hasattr( aq_base(content), 'allow_discussion' ):
            return bool(content.allow_discussion)
        typeInfo = content.getTypeInfo()
        if typeInfo:
            return bool( typeInfo.allowDiscussion() )
        return False

    #
    #   Utility methods
    #
    security.declarePrivate( '_createDiscussionFor' )
    def _createDiscussionFor( self, content ):
        """ Create DiscussionItemContainer for content, if allowed.
        """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed

        content.talkback = DiscussionItemContainer()
        return content.talkback

InitializeClass( DiscussionTool )
registerToolInterface('portal_discussion', IDiscussionTool)



##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Formlib form base classes.

$Id: form.py 76996 2007-06-24 00:18:49Z hannosch $
"""

from datetime import datetime
from sets import Set

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.formlib.formbase import PageDisplayForm
from Products.Five.formlib.formbase import PageForm
from zope.datetime import parseDatetimetz
from zope.formlib import form
from zope.i18n.interfaces import IUserPreferredLanguages
from zope.i18n.locales import LoadLocaleError
from zope.i18n.locales import locales
from ZTUtils import make_query

from Products.CMFDefault.utils import Message as _
from Products.CMFDefault.utils import translate
from Products.CMFDefault.browser.utils import ViewBase


# from zope.publisher.http.HTTPRequest
def getLocale(request):
    envadapter = IUserPreferredLanguages(request, None)
    if envadapter is None:
        return None

    langs = envadapter.getPreferredLanguages()
    for httplang in langs:
        parts = (httplang.split('-') + [None, None])[:3]
        try:
            return locales.getLocale(*parts)
        except LoadLocaleError:
            # Just try the next combination
            pass
    else:
        # No combination gave us an existing locale, so use the default,
        # which is guaranteed to exist
        return locales.getLocale(None, None, None)


class EditFormBase(PageForm, ViewBase):

    template = ViewPageTemplateFile('editform.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.request.locale = getLocale(request)

    def _setRedirect(self, provider_id, action_path, keys=''):
        provider = self._getTool(provider_id)
        try:
            target = provider.getActionInfo(action_path, self.context)['url']
        except ValueError:
            target = self._getPortalURL()

        kw = {}
        if self.status:
            message = translate(self.status, self.context)
            if isinstance(message, unicode):
                message = message.encode(self._getBrowserCharset())
            kw['portal_status_message'] = message
        for k in keys.split(','):
            k = k.strip()
            v = self.request.form.get(k, None)
            if v:
                kw[k] = v

        query = kw and ( '?%s' % make_query(kw) ) or ''
        self.request.RESPONSE.redirect( '%s%s' % (target, query) )

        return ''

    def handle_failure(self, action, data, errors):
        if self.status:
            message = translate(self.status, self.context)
            self.request.other['portal_status_message'] = message


class ContentEditFormBase(EditFormBase):

    actions = form.Actions(
        form.Action(
            name='change',
            label=_(u'Change'),
            validator='handle_validate',
            success='handle_change_success',
            failure='handle_failure'),
        form.Action(
            name='change_and_view',
            label=_(u'Change and View'),
            validator='handle_validate',
            success='handle_change_and_view_success',
            failure='handle_failure'))

    description = u''

    def setUpWidgets(self, ignore_request=False):
        self.adapters = {}
        self.widgets = form.setUpEditWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            adapters=self.adapters, ignore_request=ignore_request
            )

    @property
    def label(self):
        obj_type = translate(self.context.Type(), self.context)
        return _(u'Edit ${obj_type}', mapping={'obj_type': obj_type})

    def handle_validate(self, action, data):
        if self.context.wl_isLocked():
            return (_(u'This resource is locked via webDAV.'),)
        return None

    def _handle_success(self, action, data):
        # normalize set and datetime
        for k, v in data.iteritems():
            if isinstance(v, Set):
                data[k] = set(v)
            elif isinstance(v, datetime) and v.tzname() is None:
                data[k] = parseDatetimetz(str(v))
        changed = form.applyChanges(self.context, self.form_fields, data,
                                    self.adapters)
        if changed:
            self.context.reindexObject()
            obj_type = translate(self.context.Type(), self.context)
            self.status = _(u'${obj_type} changed.',
                            mapping={'obj_type': obj_type})
        else:
            self.status = _(u'Nothing to change.')
        return changed

    def handle_change_success(self, action, data):
        self._handle_success(action, data)
        return self._setRedirect('portal_types', 'object/edit')

    def handle_change_and_view_success(self, action, data):
        self._handle_success(action, data)
        return self._setRedirect('portal_types', 'object/view')


class DisplayFormBase(PageDisplayForm, ViewBase):

    template = ViewPageTemplateFile('viewform.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.request.locale = getLocale(request)

    @property
    def label(self):
        return self.context.Type()



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" News content object.

$Id: NewsItem.py 77186 2007-06-28 19:06:19Z yuppie $
"""

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from zope.component.factory import Factory
from zope.interface import implements

from Document import Document
from interfaces import IMutableNewsItem
from interfaces import INewsItem
from permissions import ModifyPortalContent


def addNewsItem( self
               , id
               , title=''
               , description=''
               , text=''
               , text_format=''
               ):
    """Add a NewsItem.
    """
    o=NewsItem( id=id
              , title=title
              , description=description
              , text=text
              , text_format=text_format
              )
    self._setObject(id, o)


class NewsItem(Document):

    """A News Item.
    """

    implements(IMutableNewsItem, INewsItem)
    __implements__ = Document.__implements__  # redundant, but explicit

    text_format = 'html'

    security = ClassSecurityInfo()

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit( self, text, description=None, text_format=None ):
        """Edit the News Item.
        """
        if text_format is None:
            text_format = getattr(self, 'text_format', 'structured-text')
        if description is not None:
            self.setDescription( description )
        Document.edit( self, text_format, text )

InitializeClass(NewsItem)

NewsItemFactory = Factory(NewsItem)



## Script (Python) "enableSyndication"
##title=Enable Syndication for a resource
##parameters=

if context.portal_syndication.isSiteSyndicationAllowed():
    context.portal_syndication.enableSyndication(context)
    msg = 'Syndication+Enabled'
else:
    msg = 'Syndication+Not+Allowed'

target = '%s/synPropertiesForm' % context.absolute_url()
qs = 'portal_status_message=%s' % msg

return context.REQUEST.RESPONSE.redirect('%s?%s' % (target, qs))



##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""CMFDefault browser tests.

$Id: tests.py 72362 2007-02-04 15:31:51Z yuppie $
"""

import unittest
from Testing import ZopeTestCase
from zope.testing import doctest

from Products.CMFDefault.testing import FunctionalLayer


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(doctest.DocFileSuite('folder.txt',
                                    optionflags=doctest.NORMALIZE_WHITESPACE))
    s = ZopeTestCase.FunctionalDocFileSuite('metadata.txt')
    s.layer = FunctionalLayer
    suite.addTest(s)
    s = ZopeTestCase.FunctionalDocFileSuite('document.txt')
    s.layer = FunctionalLayer
    suite.addTest(s)
    return suite

if __name__ == '__main__':
    from Products.CMFCore.testing import run
    run(test_suite())



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Portal class

$Id: Portal.py 39562 2005-10-23 09:46:52Z yuppie $
"""

from Globals import InitializeClass

from Products.CMFCore.PortalObject import PortalObjectBase

from DublinCore import DefaultDublinCoreImpl
from permissions import AddPortalContent
from permissions import AddPortalFolders
from permissions import ListPortalMembers
from permissions import ReplyToItem
from permissions import View


class CMFSite(PortalObjectBase, DefaultDublinCoreImpl):

    """
        The *only* function this class should have is to help in the setup
        of a new CMFSite.  It should not assist in the functionality at all.
    """
    meta_type = 'CMF Site'

    _properties = (
        {'id':'title', 'type':'string', 'mode': 'w'},
        {'id':'description', 'type':'text', 'mode': 'w'},
        )
    title = ''
    description = ''

    __ac_permissions__=( ( AddPortalContent, () )
                       , ( AddPortalFolders, () )
                       , ( ListPortalMembers, () )
                       , ( ReplyToItem, () )
                       , ( View, ('isEffective',) )
                       )

    def __init__( self, id, title='' ):
        PortalObjectBase.__init__( self, id, title )
        DefaultDublinCoreImpl.__init__( self )

    def isEffective( self, date ):
        """
            Override DefaultDublinCoreImpl's test, since we are always viewable.
        """
        return 1

    def reindexObject( self, idxs=[] ):
        """
            Override DefaultDublinCoreImpl's method (so that we can play
            in 'editMetadata').
        """
        pass

InitializeClass(CMFSite)



##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for Link module.

$Id: test_Link.py 68344 2006-05-29 14:19:25Z yuppie $
"""

import unittest
import Testing

from re import compile

from Products.CMFCore.testing import ConformsToContent
from Products.CMFCore.tests.base.content import BASIC_RFC822
from Products.CMFCore.tests.base.content import RFC822_W_CONTINUATION


class LinkTests(ConformsToContent, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFDefault.Link import Link

        return Link

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def canonTest(self, table):
        for orig, wanted in table.items():
            # test with constructor
            d = self._makeOne('foo', remote_url=orig)
            self.assertEqual(d.getRemoteUrl(), wanted)
            # test with edit method too
            d = self._makeOne('bar')
            d.edit(orig)
            self.assertEqual(d.getRemoteUrl(), wanted)

    def test_z3interfaces(self):
        from zope.interface.verify import verifyClass
        from Products.CMFDefault.interfaces import ILink
        from Products.CMFDefault.interfaces import IMutableLink

        verifyClass(ILink, self._getTargetClass())
        verifyClass(IMutableLink, self._getTargetClass())

    def test_Empty( self ):
        d = self._makeOne('foo')
        self.assertEqual( d.Title(), '' )
        self.assertEqual( d.Description(), '' )
        self.assertEqual( d.getRemoteUrl(), '' )
        self.assertEqual( d.format, 'text/url' )
        self.assertEqual( d.URL_FORMAT, 'text/url')

        d = self._makeOne('foo', remote_url='bar')
        d.edit('')
        self.assertEqual(d.getRemoteUrl(), '')

        d = self._makeOne('foo', remote_url='http://')
        self.assertEqual(d.getRemoteUrl(), '')

        d = self._makeOne('foo', remote_url='http:')
        self.assertEqual(d.getRemoteUrl(), '')

    def test_RFC822(self):
        d = self._makeOne('foo')
        d._writeFromPUT( body=BASIC_RFC822 )

        self.assertEqual( d.Title(), 'Zope Community' )
        self.assertEqual( d.Description()
                        , 'Link to the Zope Community website.' )
        self.assertEqual( len(d.Subject()), 3 )
        self.assertEqual( d.getRemoteUrl(), 'http://www.zope.org' )

    def test_RFC822_w_Continuation(self):
        d = self._makeOne('foo')
        d._writeFromPUT( body=RFC822_W_CONTINUATION )
        rnlinesplit = compile( r'\r?\n?' )
        desc_lines = rnlinesplit.split( d.Description() )

        self.assertEqual( d.Title(), 'Zope Community' )
        self.assertEqual( desc_lines[0]
                        , 'Link to the Zope Community website,' )
        self.assertEqual( desc_lines[1]
                        , ' including hundreds of contributed Zope products.' )
        self.assertEqual( len(d.Subject()), 3 )
        self.assertEqual( d.getRemoteUrl(), 'http://www.zope.org' )

    def test_PutWithoutMetadata(self):
        d = self._makeOne('foo')
        d._writeFromPUT( body='' )
        self.assertEqual( d.Title(), '' )
        self.assertEqual( d.Format(), 'text/url' )
        self.assertEqual( d.Description(), '' )
        self.assertEqual( d.Subject(), () )
        self.assertEqual( d.Contributors(), () )
        self.assertEqual( d.EffectiveDate(), 'None' )
        self.assertEqual( d.ExpirationDate(), 'None' )
        self.assertEqual( d.Language(), '' )
        self.assertEqual( d.Rights(), '' )

    def test_fixupMissingScheme(self):
        table = {
            'http://foo.com':      'http://foo.com',
            '//bar.com':           'http://bar.com',
            }
        self.canonTest(table)

    def test_keepRelativeUrl(self):
        table = {
            'baz.com':             'baz.com',
            'baz2.com/index.html': 'baz2.com/index.html',
            '/huh/zoinx.html':     '/huh/zoinx.html',
            'hmmm.com/lol.txt':    'hmmm.com/lol.txt',
            }
        self.canonTest(table)

    def test_trailingSlash(self):
        table = {
            'http://foo.com/bar/': 'http://foo.com/bar/',
            'baz.com/':            'baz.com/',
            '/baz.org/zoinx/':     '/baz.org/zoinx/',
            }
        self.canonTest(table)

    def test_otherScheme(self):
        table = {
            'mailto:user@foo.com':      'mailto:user@foo.com',
            'https://bank.com/account': 'https://bank.com/account',
            }
        self.canonTest(table)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(LinkTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for MetadataTool module.

$Id: test_MetadataTool.py 77004 2007-06-24 08:57:54Z yuppie $
"""

import unittest
import Testing

from Acquisition import aq_base


class TestMetadataElementPolicy(unittest.TestCase):

    def _getTargetClass( self ):
        from Products.CMFDefault.MetadataTool import MetadataElementPolicy
        return MetadataElementPolicy

    def _makeOne( self, *args, **kw ):
        return self._getTargetClass()( *args, **kw )

    def test_empty_single_valued( self ):
        sv_policy = self._makeOne( 0 )
        self.failIf( sv_policy.isMultiValued() )
        self.failIf( sv_policy.isRequired() )
        self.failIf( sv_policy.supplyDefault() )
        self.failIf( sv_policy.defaultValue() )
        self.failIf( sv_policy.enforceVocabulary() )
        self.failIf( sv_policy.allowedVocabulary() )

    def test_edit_single_valued( self ):
        sv_policy = self._makeOne( 0 )
        sv_policy.edit( 1, 1, 'xxx', 0, '' )
        self.failIf( sv_policy.isMultiValued() )
        self.failUnless( sv_policy.isRequired() )
        self.failUnless( sv_policy.supplyDefault() )
        self.assertEquals( sv_policy.defaultValue(), 'xxx' )
        self.failIf( sv_policy.enforceVocabulary() )
        self.failIf( sv_policy.allowedVocabulary() )

    def test_empty_multi_valued( self ):
        mv_policy = self._makeOne( 1 )
        self.failUnless( mv_policy.isMultiValued() )
        self.failIf( mv_policy.isRequired() )
        self.failIf( mv_policy.supplyDefault() )
        self.failIf( mv_policy.defaultValue() )
        self.failIf( mv_policy.enforceVocabulary() )
        self.failIf( mv_policy.allowedVocabulary() )

    def test_edit_multi_valued( self ):
        mv_policy = self._makeOne( 1 )
        mv_policy.edit( 1, 1, 'xxx', 1, ( 'xxx', 'yyy' ) )
        self.failUnless( mv_policy.isMultiValued() )
        self.failUnless( mv_policy.isRequired() )
        self.failUnless( mv_policy.supplyDefault() )
        self.assertEqual( mv_policy.defaultValue(), 'xxx' )
        self.failUnless( mv_policy.enforceVocabulary() )
        self.assertEqual( len( mv_policy.allowedVocabulary() ), 2 )
        self.failUnless( 'xxx' in mv_policy.allowedVocabulary() )
        self.failUnless( 'yyy' in mv_policy.allowedVocabulary() )


class TestElementSpec(unittest.TestCase):

    def _getTargetClass( self ):
        from Products.CMFDefault.MetadataTool import ElementSpec
        return ElementSpec

    def _makeOne( self, *args, **kw ):
        return self._getTargetClass()( *args, **kw )

    def test_empty_single_valued( self ):
        sv_spec = self._makeOne( 0 )
        self.failIf( sv_spec.isMultiValued() )
        self.assertEqual( sv_spec.getPolicy(), sv_spec.getPolicy( 'XYZ' ) )
        policies = sv_spec.listPolicies()
        self.assertEqual( len( policies ), 1 )
        self.assertEqual( policies[0][0], None )

    def test_empty_multi_valued( self ):
        mv_spec = self._makeOne( 1 )
        self.failUnless( mv_spec.isMultiValued() )
        self.assertEqual( mv_spec.getPolicy(), mv_spec.getPolicy( 'XYZ' ) )
        policies = mv_spec.listPolicies()
        self.assertEqual( len( policies ), 1 )
        self.assertEqual( policies[0][0], None )



class TestMetadataSchema(unittest.TestCase):

    def _getTargetClass( self ):
        from Products.CMFDefault.MetadataTool import MetadataSchema
        return MetadataSchema

    def _makeOne( self, *args, **kw ):
        return self._getTargetClass()( *args, **kw )


class TestMetadataTool(unittest.TestCase):

    def _getTargetClass( self ):
        from Products.CMFDefault.MetadataTool import MetadataTool
        return MetadataTool

    def _makeOne( self, *args, **kw ):
        return self._getTargetClass()( *args, **kw )

    def _makeTestObjects( self ):

        from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
        class Foo( DefaultDublinCoreImpl ):

            description = title = language = format = rights = ''
            subject = ()

            def __init__( self ):
                pass # skip DDCI's default values

            def getPortalTypeName( self ):
                return 'Foo'


        class Bar( Foo ):

            def getPortalTypeName( self ):
                return 'Bar'

        return Foo(), Bar()

    def test_z2interfaces(self):
        from Interface.Verify import verifyClass
        from Products.CMFCore.interfaces.portal_metadata \
                import portal_metadata as IMetadataTool

        verifyClass(IMetadataTool, self._getTargetClass())

    def test_z3interfaces(self):
        from zope.interface.verify import verifyClass
        from Products.CMFCore.interfaces import IMetadataTool

        verifyClass(IMetadataTool, self._getTargetClass())

    def test_empty( self ):
        from Products.CMFDefault.MetadataTool import _DCMI_ELEMENT_SPECS

        tool = self._makeOne()
        self.failIf( tool.getPublisher() )
        self.assertEqual( tool.getFullName( 'foo' ), 'foo' )

        dcmi = tool.DCMI
        specs = list( dcmi.DCMI.listElementSpecs() )
        defaults = list( _DCMI_ELEMENT_SPECS )
        specs.sort(); defaults.sort()

        self.assertEqual( len( specs ), len( defaults ) )
        for i in range( len( specs ) ):
            self.assertEqual( specs[i][0], defaults[i][0] )
            self.assertEqual( specs[i][1].isMultiValued(), defaults[i][1] )
            policies = specs[i][1].listPolicies()
            self.assertEqual( len( policies ), 1 )
            self.failUnless( policies[0][0] is None )

        self.failIf( dcmi.getElementSpec( 'Title' ).isMultiValued() )
        self.failIf( dcmi.getElementSpec( 'Description' ).isMultiValued() )
        self.failUnless( dcmi.getElementSpec( 'Subject' ).isMultiValued() )
        self.failIf( dcmi.getElementSpec( 'Format' ).isMultiValued() )
        self.failIf( dcmi.getElementSpec( 'Language' ).isMultiValued() )
        self.failIf( dcmi.getElementSpec( 'Rights' ).isMultiValued() )

        try:
            dummy = dcmi.getElementSpec( 'Foo' )
        except KeyError:
            pass
        else:
            self.failUnless( 0, "Expected KeyError" )

        self.failIf( dcmi.listAllowedSubjects() )
        self.failIf( dcmi.listAllowedFormats() )
        self.failIf( dcmi.listAllowedLanguages() )
        self.failIf( dcmi.listAllowedRights() )

    def test_DCMI_addElementSpec( self ):
        from Products.CMFDefault.MetadataTool import _DCMI_ELEMENT_SPECS

        tool = self._makeOne()
        dcmi = tool.DCMI
        dcmi.addElementSpec( 'Rating', 1 )
        self.assertEqual( len( dcmi.listElementSpecs() )
                        , len( _DCMI_ELEMENT_SPECS ) + 1 )
        rating = dcmi.getElementSpec( 'Rating' )
        self.failUnless( rating.isMultiValued() )

    def test_DCMI_removeElementSpec( self ):
        from Products.CMFDefault.MetadataTool import _DCMI_ELEMENT_SPECS

        tool = self._makeOne()
        dcmi = tool.DCMI
        dcmi.removeElementSpec( 'Rights' )

        self.assertEqual( len( dcmi.listElementSpecs() )
                        , len( _DCMI_ELEMENT_SPECS ) - 1
                        )

        self.assertRaises( KeyError, dcmi.getElementSpec, 'Rights' )
        self.assertRaises( KeyError, dcmi.removeElementSpec, 'Foo' )

    def test_simplePolicies( self ):

        tool = self._makeOne()
        dcmi = tool.DCMI
        tSpec = dcmi.getElementSpec( 'Title' )

        # Fetch default policy.
        tDef  = tSpec.getPolicy()
        self.failIf( tDef.isRequired() )
        self.failIf( tDef.supplyDefault() )
        self.failIf( tDef.defaultValue() )

        # Fetch (default) policy for a type.
        tDoc  = tSpec.getPolicy( 'Document' )
        self.assertEqual(aq_base(tDoc), aq_base(tDef))

        # Changing default changes policies found from there.
        tDef.edit( 1, 1, 'xyz', 0, () )
        self.failUnless( tDef.isRequired() )
        self.failUnless( tDef.supplyDefault() )
        self.assertEqual( tDef.defaultValue(), 'xyz' )
        self.failUnless( tDoc.isRequired() )
        self.failUnless( tDoc.supplyDefault() )
        self.assertEqual( tDoc.defaultValue(), 'xyz' )

        tSpec.addPolicy( 'Document' )
        self.assertEqual( len( tSpec.listPolicies() ), 2 )

        tDoc  = tSpec.getPolicy( 'Document' )
        self.assertNotEqual(aq_base(tDoc), aq_base(tDef))
        self.failIf( tDoc.isRequired() )
        self.failIf( tDoc.supplyDefault() )
        self.failIf( tDoc.defaultValue() )

        tSpec.removePolicy( 'Document' )
        tDoc  = tSpec.getPolicy( 'Document' )
        self.assertEqual(aq_base(tDoc), aq_base(tDef))
        self.failUnless( tDoc.isRequired() )
        self.failUnless( tDoc.supplyDefault() )
        self.assertEqual( tDoc.defaultValue(), 'xyz' )

    def test_multiValuedPolicies( self ):

        tool = self._makeOne()
        dcmi = tool.DCMI
        sSpec = dcmi.getElementSpec( 'Subject' )

        # Fetch default policy.
        sDef  = sSpec.getPolicy()
        self.failIf( sDef.isRequired() )
        self.failIf( sDef.supplyDefault() )
        self.failIf( sDef.defaultValue() )
        self.failIf( sDef.enforceVocabulary() )
        self.failIf( sDef.allowedVocabulary() )

        # Fetch (default) policy for a type.
        sDoc  = sSpec.getPolicy( 'Document' )
        self.assertEqual(aq_base(sDoc), aq_base(sDef))

        # Changing default changes policies found from there.
        sDef.edit( 1, 1, 'xyz', 1, ( 'foo', 'bar' ) )
        self.failUnless( sDef.isRequired() )
        self.failUnless( sDef.supplyDefault() )
        self.assertEqual( sDef.defaultValue(), 'xyz' )
        self.failUnless( sDoc.isRequired() )
        self.failUnless( sDoc.supplyDefault() )
        self.assertEqual( sDoc.defaultValue(), 'xyz' )
        self.failUnless( sDef.enforceVocabulary() )
        self.assertEqual( len( sDef.allowedVocabulary() ), 2 )
        self.failUnless( 'foo' in sDef.allowedVocabulary() )
        self.failUnless( 'bar' in sDef.allowedVocabulary() )
        self.failUnless( sDoc.enforceVocabulary() )
        self.assertEqual( len( sDoc.allowedVocabulary() ), 2 )
        self.failUnless( 'foo' in sDoc.allowedVocabulary() )
        self.failUnless( 'bar' in sDoc.allowedVocabulary() )

        sSpec.addPolicy( 'Document' )
        self.assertEqual( len( sSpec.listPolicies() ), 2 )

        sDoc  = sSpec.getPolicy( 'Document' )
        self.assertNotEqual(aq_base(sDoc), aq_base(sDef))
        self.failIf( sDoc.isRequired() )
        self.failIf( sDoc.supplyDefault() )
        self.failIf( sDoc.defaultValue() )
        self.failIf( sDoc.enforceVocabulary() )
        self.failIf( sDoc.allowedVocabulary() )

        sSpec.removePolicy( 'Document' )
        sDoc  = sSpec.getPolicy( 'Document' )
        self.assertEqual(aq_base(sDoc), aq_base(sDef))
        self.failUnless( sDoc.isRequired() )
        self.failUnless( sDoc.supplyDefault() )
        self.assertEqual( sDoc.defaultValue(), 'xyz' )
        self.failUnless( sDoc.enforceVocabulary() )
        self.assertEqual( len( sDoc.allowedVocabulary() ), 2 )
        self.failUnless( 'foo' in sDoc.allowedVocabulary() )
        self.failUnless( 'bar' in sDoc.allowedVocabulary() )

    def test_vocabularies( self ):
        tool = self._makeOne()
        dcmi = tool.DCMI
        fSpec   = dcmi.getElementSpec( 'Format' )
        fDef    = fSpec.getPolicy()
        formats = ( 'text/plain', 'text/html' )
        fDef.edit( 0, 0, '', 0, ( 'text/plain', 'text/html' ) )
        self.assertEqual( tool.listAllowedFormats(), formats )

        foo, bar = self._makeTestObjects()

        self.assertEqual( tool.listAllowedFormats( foo ), formats )

        fSpec.addPolicy( 'Foo' )
        self.failIf( tool.listAllowedFormats( foo ) )

        foo_formats = ( 'image/jpeg', 'image/gif', 'image/png' )
        fFoo        = fSpec.getPolicy( 'Foo' )
        fFoo.edit( 0, 0, '', 0, foo_formats )
        self.assertEqual( tool.listAllowedFormats( foo ), foo_formats )

    def test_initialValues_defaults( self ):
        tool = self._makeOne()
        foo, bar = self._makeTestObjects()
        self.failIf( foo.Title() )
        self.failIf( foo.Description() )
        self.failIf( foo.Subject() )
        self.failIf( foo.Format(), foo.Format() )
        self.failIf( foo.Language() )
        self.failIf( foo.Rights() )

        tool.setInitialMetadata( foo )
        self.failIf( foo.Title() )
        self.failIf( foo.Description() )
        self.failIf( foo.Subject() )
        self.failIf( foo.Format() )
        self.failIf( foo.Language() )
        self.failIf( foo.Rights() )

    def test_initialValues_implicit( self ):
        # Test default policy.
        tool = self._makeOne()
        dcmi = tool.DCMI
        foo, bar = self._makeTestObjects()
        fSpec   = dcmi.getElementSpec( 'Format' )
        fPolicy = fSpec.getPolicy()
        fPolicy.edit( 0, 1, 'text/plain', 0, () )
        tool.setInitialMetadata( foo )
        self.failIf( foo.Title() )
        self.failIf( foo.Description() )
        self.failIf( foo.Subject() )
        self.assertEqual( foo.Format(), 'text/plain' )
        self.failIf( foo.Language() )
        self.failIf( foo.Rights() )

    def test_initialValues_explicit_raises_if_constraint_fails( self ):
        from Products.CMFDefault.exceptions import MetadataError

        # Test type-specific policy.
        tool = self._makeOne()
        dcmi = tool.DCMI
        foo, bar = self._makeTestObjects()
        tSpec   = dcmi.getElementSpec( 'Title' )
        tSpec.addPolicy( 'Foo' )
        tPolicy = tSpec.getPolicy( foo.getPortalTypeName() )
        tPolicy.edit( 1, 0, '', 0, () )

        self.assertRaises( MetadataError, tool.setInitialMetadata, foo )

    def test_initialValues_explicit_mutliple_types( self ):
        tool = self._makeOne()
        dcmi = tool.DCMI
        foo, bar = self._makeTestObjects()
        foo.setTitle( 'Foo title' )

        fSpec   = dcmi.getElementSpec( 'Format' )
        fSpec.addPolicy( foo.getPortalTypeName() )
        fPolicy = fSpec.getPolicy( foo.getPortalTypeName() )
        fPolicy.edit( 0, 1, 'text/plain', 0, () )

        tool.setInitialMetadata( foo )
        self.assertEqual( foo.Title(), 'Foo title' )
        self.failIf( foo.Description() )
        self.failIf( foo.Subject() )
        self.assertEqual( foo.Format(), 'text/plain' )
        self.failIf( foo.Language() )
        self.failIf( foo.Rights() )

        #   Ensure Foo's policy doesn't interfere with other types.
        tool.setInitialMetadata( bar )
        self.failIf( bar.Title() )
        self.failIf( bar.Description() )
        self.failIf( bar.Subject() )
        self.assertEqual( bar.Format(), '' )
        self.failIf( bar.Language() )
        self.failIf( bar.Rights() )

    def test_validation( self ):
        from Products.CMFDefault.exceptions import MetadataError

        tool = self._makeOne()
        foo, bar = self._makeTestObjects()
        tool.setInitialMetadata( foo )
        tool.validateMetadata( foo )

        dcmi = tool.DCMI
        tSpec   = dcmi.getElementSpec( 'Title' )
        tSpec.addPolicy( 'Foo' )
        tPolicy = tSpec.getPolicy( foo.getPortalTypeName() )
        tPolicy.edit( 1, 0, '', 0, () )

        self.assertRaises( MetadataError, tool.validateMetadata, foo )

        foo.setTitle( 'Foo title' )
        tool.validateMetadata( foo )

    def test_addSchema_normal( self ):
        from Products.CMFDefault.MetadataTool import MetadataSchema

        tool = self._makeOne()
        before = tool.listSchemas()
        self.assertEqual( len( before ), 1 )
        self.assertEqual( before[0][0], 'DCMI' )
        self.failUnless( isinstance( before[0][1], MetadataSchema ) )

        tool.addSchema( 'Arbitrary' )

        after = tool.listSchemas()
        self.assertEqual( len( after ), 2 )
        self.assertEqual( after[0][0], 'DCMI' )
        self.failUnless( isinstance( after[0][1], MetadataSchema ) )
        self.assertEqual( after[1][0], 'Arbitrary' )
        self.failUnless( isinstance( after[1][1], MetadataSchema ) )

    def test_addSchema_duplicate( self ):
        tool = self._makeOne()
        before = tool.listSchemas()
        tool.addSchema( 'Arbitrary' )
        self.assertRaises( KeyError, tool.addSchema, 'Arbitrary' )
        self.assertRaises( KeyError, tool.addSchema, 'DCMI' )

    def test_removeSchema_normal( self ):
        tool = self._makeOne()
        before = tool.listSchemas()
        self.assertEqual( len( before ), 1 )
        self.assertEqual( before[0][0], 'DCMI' )

        tool.addSchema( 'Arbitrary' )
        tool.addSchema( 'Beneficent' )
        tool.addSchema( 'Grouchy' )

        middle = tool.listSchemas()
        self.assertEqual( len( middle ), 4 )
        self.assertEqual( middle[0][0], 'DCMI' )
        self.assertEqual( middle[1][0], 'Arbitrary' )
        self.assertEqual( middle[2][0], 'Beneficent' )
        self.assertEqual( middle[3][0], 'Grouchy' )

        tool.removeSchema( 'Beneficent' )

        after = tool.listSchemas()
        self.assertEqual( len( after ), 3 )
        self.assertEqual( after[0][0], 'DCMI' )
        self.assertEqual( after[1][0], 'Arbitrary' )
        self.assertEqual( after[2][0], 'Grouchy' )

    def test_removeSchema_invalid( self ):
        tool = self._makeOne()
        self.assertRaises( KeyError, tool.removeSchema, 'DCMI' )
        tool.addSchema( 'Arbitrary' )
        tool.removeSchema( 'Arbitrary' )
        self.assertRaises( KeyError, tool.removeSchema, 'Arbitrary' )

    def test_migration( self ):
        # Test that we forward-migrate old-style DCMI policies.
        from Products.CMFDefault.MetadataTool import ElementSpec

        tool = self._makeOne()
        tool.element_specs = { 'Title' : ElementSpec( 0 )
                             , 'Description' : ElementSpec( 0 )
                             , 'Subject' : ElementSpec( 1 )
                             , 'Format' : ElementSpec( 0 )
                             , 'Language' : ElementSpec( 0 )
                             , 'Rights' : ElementSpec( 0 )
                             }
        subj = tool.element_specs[ 'Subject' ]
        subj.addPolicy( 'Foo' )
        subj.getPolicy( 'Foo' ).edit( False
                                    , False
                                    , None
                                    , True
                                    , ( 'bar', 'baz' )
                                    )

        dcmi = tool.DCMI

        self.assertEqual( dcmi.getId(), 'DCMI' )

        # Accessing the DCMI property converts and clears 'element_specs'
        self.assertRaises(AttributeError, lambda: tool.element_specs )

        subj2 = dcmi.getElementSpec( 'Subject' )
        subj_default = subj2.getPolicy( None )
        subj_foo = subj2.getPolicy( 'Foo' )

        self.assertEqual( subj_foo.isRequired(), False )
        self.assertEqual( subj_foo.supplyDefault(), False )
        self.assertEqual( subj_foo.defaultValue(), None )
        self.assertEqual( subj_foo.enforceVocabulary(), True )
        self.assertEqual( len( subj_foo.allowedVocabulary() ), 2 )
        self.failUnless( 'bar' in subj_foo.allowedVocabulary() )
        self.failUnless( 'baz' in subj_foo.allowedVocabulary() )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(TestMetadataElementPolicy),
        unittest.makeSuite(TestElementSpec),
        unittest.makeSuite(TestMetadataTool),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" CMFDefault.interfaces package.

$Id: __init__.py 71020 2006-11-01 15:42:13Z yuppie $
"""

from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from _content import *
from _tools import *


class ICMFDefaultSkin(IDefaultBrowserLayer):

    """CMF default skin.
    """


# BBB: will be removed in CMF 2.2
#      create zope2 interfaces
from Interface.bridge import createZope3Bridge
import Document
import portal_membership

createZope3Bridge(IDocument, Document, 'IDocument')
createZope3Bridge(IMutableDocument, Document, 'IMutableDocument')
createZope3Bridge(IMembershipTool, portal_membership, 'portal_membership')

del createZope3Bridge



## Script (Python) "getBatchItemInfos"
##parameters=batch_obj
##title=
##
from Products.CMFCore.utils import getToolByName

utool = getToolByName(script, 'portal_url')
portal_url = utool()

items = []

for item in batch_obj:
    item_description = item.Description()
    item_icon = item.getIcon(1)
    item_title = item.Title()
    item_type = remote_type = item.Type()
    if item_type == 'Favorite' and not item_icon == 'p_/broken':
        item = item.getObject()
        item_description = item_description or item.Description()
        item_title = item_title or item.Title()
        remote_type = item.Type()
    is_file = remote_type in ('File', 'Image')
    is_link = remote_type == 'Link'
    items.append( { 'description': item_description,
                    'format': is_file and item.Format() or '',
                    'icon': item_icon and
                            ( '%s/%s' % (portal_url, item_icon) ) or '',
                    'size': is_file and
                            '%0.0f kb' % ( item.get_size() / 1024.0 ) or '',
                    'title': item_title,
                    'type': item_type,
                    'url': is_link and item.getRemoteUrl() or
                            item.absolute_url() } )

return tuple(items)



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for Favorites.

$Id: test_Favorite.py 77113 2007-06-26 20:36:26Z yuppie $
"""

import unittest
import Testing

from zope.component import getSiteManager
from zope.testing.cleanup import cleanUp

from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.testing import ConformsToContent
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.dummy import DummyTool


class FavoriteTests(ConformsToContent, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFDefault.Favorite import Favorite

        return Favorite

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def setUp(self):
        sm = getSiteManager()
        self.site = DummySite('site')
        sm.registerUtility(self.site, ISiteRoot)
        self.site._setObject( 'portal_membership', DummyTool() )
        self.site._setObject( 'portal_url', DummyTool() )

    def tearDown(self):
        cleanUp()

    def test_z3interfaces(self):
        from zope.interface.verify import verifyClass
        from Products.CMFDefault.interfaces import IFavorite
        from Products.CMFDefault.interfaces import ILink
        from Products.CMFDefault.interfaces import IMutableFavorite
        from Products.CMFDefault.interfaces import IMutableLink

        verifyClass(IFavorite, self._getTargetClass())
        verifyClass(ILink, self._getTargetClass())
        verifyClass(IMutableFavorite, self._getTargetClass())
        verifyClass(IMutableLink, self._getTargetClass())

    def test_Empty( self ):
        utool = self.site.portal_url
        f = self.site._setObject('foo', self._makeOne('foo'))

        self.assertEqual( f.getId(), 'foo' )
        self.assertEqual( f.Title(), '' )
        self.assertEqual( f.Description(), '' )
        self.assertEqual( f.getRemoteUrl(), utool.root )
        self.assertEqual( f.getObject(), self.site )
        self.assertEqual( f.getIcon(), self.site.getIcon() )
        self.assertEqual( f.getIcon(1), self.site.getIcon(1) )

    def test_CtorArgs( self ):
        utool = self.site.portal_url
        self.assertEqual( self._makeOne( 'foo'
                                       , title='Title'
                                       ).Title(), 'Title' )

        self.assertEqual( self._makeOne( 'bar'
                                       , description='Description'
                                       ).Description(), 'Description' )

        baz = self.site._setObject('foo',
                                self._makeOne('baz', remote_url='portal_url'))
        self.assertEqual( baz.getObject(), utool )
        self.assertEqual( baz.getRemoteUrl()
                        , '%s/portal_url' % utool.root )
        self.assertEqual( baz.getIcon(), utool.getIcon() )

    def test_edit( self ):
        utool = self.site.portal_url
        f = self.site._setObject('foo', self._makeOne('foo'))
        f.edit( 'portal_url' )
        self.assertEqual( f.getObject(), utool )
        self.assertEqual( f.getRemoteUrl()
                        , '%s/portal_url' % utool.root )
        self.assertEqual( f.getIcon(), utool.getIcon() )

    def test_editEmpty( self ):
        utool = self.site.portal_url
        f = self.site._setObject('gnnn', self._makeOne('gnnn'))
        f.edit( '' )
        self.assertEqual( f.getObject(), self.site )
        self.assertEqual( f.getRemoteUrl(), utool.root )
        self.assertEqual( f.getIcon(), self.site.getIcon() )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(FavoriteTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for DiscussionTool module.

$Id: test_DiscussionTool.py 81923 2007-11-18 22:51:13Z jens $
"""

import unittest
import Testing

from zope.component import getSiteManager
from zope.testing.cleanup import cleanUp

from Products.CMFCore.interfaces import IDiscussionTool
from Products.CMFCore.tests.base.dummy import DummyFolder
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.dummy import DummyTool
from Products.CMFCore.tests.base.testcase import SecurityTest


class DiscussionToolTests(SecurityTest):

    def _makeOne(self, *args, **kw):
        from Products.CMFDefault.DiscussionTool import DiscussionTool

        return DiscussionTool(*args, **kw)

    def setUp(self):
        SecurityTest.setUp(self)
        self.site = DummySite('site')
        sm = getSiteManager()
        self.site._setObject( 'portal_discussion', self._makeOne() )
        sm.registerUtility(self.site.portal_discussion, IDiscussionTool)
        self.site._setObject( 'portal_types', DummyTool() )

    def tearDown(self):
        cleanUp()
        SecurityTest.tearDown(self)

    def test_z2interfaces(self):
        from Interface.Verify import verifyClass
        from Products.CMFCore.interfaces.portal_discussion \
                import portal_discussion as IDiscussionTool
        from Products.CMFDefault.DiscussionTool import DiscussionTool

        verifyClass(IDiscussionTool, DiscussionTool)

    def test_z3interfaces(self):
        from zope.interface.verify import verifyClass
        from Products.CMFCore.interfaces import IDiscussionTool
        from Products.CMFDefault.DiscussionTool import DiscussionTool

        verifyClass(IDiscussionTool, DiscussionTool)

    def test_overrideDiscussionFor(self):
        dtool = self.site.portal_discussion
        foo = self.site._setObject( 'foo', DummyFolder() )
        baz = foo._setObject( 'baz', DummyFolder() )

        dtool.overrideDiscussionFor(foo, 1)
        self.failUnless( hasattr(foo.aq_base, 'allow_discussion') )
        try:
            dtool.overrideDiscussionFor(baz, None)
        except KeyError:
            self.fail('CMF Collector issue #201 (acquisition bug): '
                      'KeyError raised')
        dtool.overrideDiscussionFor(foo, None)
        self.failIf( hasattr(foo.aq_base, 'allow_discussion') )

        # https://bugs.launchpad.net/zope-cmf/+bug/162532: Don't break
        # if allow_discussion only exists at the class level
        class DummyContent:
            allow_discussion = False
            def getId(self): return 'dummy'

        dummy = DummyContent()
        try:
            dtool.overrideDiscussionFor(dummy, None)
        except AttributeError:
            self.fail('Launchpad issue 162532: AttributeError raised')

    def test_isDiscussionAllowedFor(self):
        # Test for Collector issue #398 (allow_discussion wrongly
        # acquired and used from parent)
        dtool = self.site.portal_discussion
        foo = self.site._setObject( 'foo', DummyFolder() )
        baz = foo._setObject( 'baz', DummyFolder() )
        dtool.overrideDiscussionFor(foo, 1)

        self.failIf(dtool.isDiscussionAllowedFor(baz))

    def test_getDiscussionFor(self):
        dtool = self.site.portal_discussion
        foo = self.site._setObject( 'foo', DummyFolder() )
        foo.allow_discussion = 1
        baz = foo._setObject( 'baz', DummyFolder() )
        baz.allow_discussion = 1

        self.failIf( hasattr(foo.aq_base, 'talkback') )
        talkback = dtool.getDiscussionFor(foo)
        self.failUnless( hasattr(foo.aq_base, 'talkback') )
        self.failIf( hasattr(baz.aq_base, 'talkback') )
        talkback = dtool.getDiscussionFor(baz)
        self.failUnless( hasattr(baz.aq_base, 'talkback'),
                         'CMF Collector issue #119 (acquisition bug): '
                         'talkback not created' )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(DiscussionToolTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser views for news items.

$Id: newsitem.py 72362 2007-02-04 15:31:51Z yuppie $
"""

from zope.component import adapts
from zope.formlib import form
from zope.interface import implements
from zope.interface import Interface
from zope.schema import Choice
from zope.schema import Text
from zope.schema import TextLine

from Products.CMFDefault.formlib.form import ContentEditFormBase
from Products.CMFDefault.formlib.schema import ProxyFieldProperty
from Products.CMFDefault.formlib.schema import SchemaAdapterBase
from Products.CMFDefault.formlib.widgets import ChoiceRadioWidget
from Products.CMFDefault.formlib.widgets import TextInputWidget
from Products.CMFDefault.interfaces import IMutableNewsItem
from Products.CMFDefault.utils import Message as _


class INewsItemSchema(Interface):

    """Schema for news item views.
    """

    title = TextLine(
        title=_(u'Title'),
        readonly=True)

    text_format = Choice(
        title=_(u'Format'),
        vocabulary='cmf.AvailableTextFormats')

    description = Text(
        title=_(u'Lead-in'),
        required=False,
        missing_value=u'')

    text = Text(
        title=_(u'Body'),
        required=False,
        missing_value=u'')


class NewsItemSchemaAdapter(SchemaAdapterBase):

    """Adapter for IMutableNewsItem.
    """

    adapts(IMutableNewsItem)
    implements(INewsItemSchema)

    title = ProxyFieldProperty(INewsItemSchema['title'], 'Title')
    text_format = ProxyFieldProperty(INewsItemSchema['text_format'])
    description = ProxyFieldProperty(INewsItemSchema['description'],
                                     'Description', 'setDescription')
    text = ProxyFieldProperty(INewsItemSchema['text'],
                              'EditableBody', '_edit')


class NewsItemEditView(ContentEditFormBase):

    """Edit view for INewsItem.
    """

    form_fields = form.FormFields(INewsItemSchema)
    form_fields['text_format'].custom_widget = ChoiceRadioWidget
    form_fields['description'].custom_widget = TextInputWidget
    form_fields['text'].custom_widget = TextInputWidget

    def setUpWidgets(self, ignore_request=False):
        super(NewsItemEditView,
              self).setUpWidgets(ignore_request=ignore_request)
        self.widgets['text_format'].orientation = 'horizontal'
        self.widgets['description'].height = 8
        self.widgets['text'].height = 16



##parameters=b_start=0, key='', reverse=0, ids=(), delta=1, items_copy='', items_cut='', items_delete='', items_new='', items_paste='', items_rename='', items_up='', items_down='', items_top='', items_bottom='', items_sort=''
##
from ZTUtils import Batch
from ZTUtils import make_query
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.permissions import AddPortalContent
from Products.CMFDefault.permissions import DeleteObjects
from Products.CMFDefault.permissions import ListFolderContents
from Products.CMFDefault.permissions import ManageProperties
from Products.CMFDefault.permissions import ViewManagementScreens
from Products.CMFDefault.utils import decode
from Products.CMFDefault.utils import html_marshal
from Products.CMFDefault.utils import Message as _

mtool = getToolByName(script, 'portal_membership')
utool = getToolByName(script, 'portal_url')
portal_url = utool()


form = context.REQUEST.form
default_target = 'object/folderContents'
default_kw = {'b_start': b_start, 'key': key, 'reverse': reverse}
if items_copy and \
        context.validateItemIds(**form) and \
        context.folder_copy_control(**form) and \
        context.setRedirect(context, default_target, **default_kw):
    return
elif items_cut and \
        context.validateItemIds(**form) and \
        context.folder_cut_control(**form) and \
        context.setRedirect(context, default_target, **default_kw):
    return
elif items_delete and \
        context.validateItemIds(**form) and \
        context.folder_delete_control(**form) and \
        context.setRedirect(context, default_target, **default_kw):
    return
elif items_new and \
        context.setRedirect(context, 'object/new'):
    return
elif items_paste and \
        context.validateClipboardData(**form) and \
        context.folder_paste_control(**form) and \
        context.setRedirect(context, default_target, **default_kw):
    return
elif items_rename and \
        context.validateItemIds(**form) and \
        context.setRedirect(context, 'object/rename_items', ids=ids,
                            **default_kw):
    return
elif items_sort and \
        context.folder_sort_control(**form) and \
        context.setRedirect(context, default_target, b_start=b_start):
    return
elif items_up and \
        context.validateItemIds(**form) and \
        context.folder_up_control(**form) and \
        context.setRedirect(context, default_target, **default_kw):
    return
elif items_down and \
        context.validateItemIds(**form) and \
        context.folder_down_control(**form) and \
        context.setRedirect(context, default_target, **default_kw):
    return
elif items_top and \
        context.validateItemIds(**form) and \
        context.folder_top_control(**form) and \
        context.setRedirect(context, default_target, **default_kw):
    return
elif items_bottom and \
        context.validateItemIds(**form) and \
        context.folder_bottom_control(**form) and \
        context.setRedirect(context, default_target, **default_kw):
    return


options = {}

options['title'] = context.Title()

items_manage_allowed = mtool.checkPermission(ViewManagementScreens, context)
items_delete_allowed = mtool.checkPermission(DeleteObjects, context)
items_add_allowed = mtool.checkPermission(AddPortalContent, context)
upitems_list_allowed = mtool.checkPermission(ListFolderContents, context,
                                             'aq_parent')
items_move_allowed = mtool.checkPermission(ManageProperties, context)

up_info = {}
if upitems_list_allowed:
    up_obj = context.aq_parent
    if hasattr(up_obj, 'portal_url'):
        up_url = up_obj.getActionInfo('object/folderContents')['url']
        up_info = { 'icon': '%s/UpFolder_icon.gif' % portal_url,
                    'id': up_obj.getId(),
                    'url': up_url }
    else:
        up_info = { 'icon': '',
                    'id': 'Root',
                    'url': '' }
options['up_info'] = up_info

target = context.getActionInfo(default_target)['url']

if not key:
    (key, reverse) = context.getDefaultSorting()
    is_default = 1
elif (key, reverse) == context.getDefaultSorting():
    is_default = 1
else:
    is_default = 0

columns = ( {'key': 'Type',
             'title': _(u'Type'),
             'width': '20',
             'colspan': '2'}
          , {'key': 'getId',
             'title': _(u'Name'),
             'width': '360',
             'colspan': None}
          , {'key': 'modified',
             'title': _(u'Last Modified'),
             'width': '180',
             'colspan': None}
          , {'key': 'position',
             'title': _(u'Position'),
             'width': '80',
             'colspan': None }
          )
for column in columns:
    if key == column['key'] and not reverse and key != 'position':
        query = make_query(key=column['key'], reverse=1)
    else:
        query = make_query(key=column['key'])
    column['url'] = '%s?%s' % (target, query)

context.filterCookie()
folderfilter = context.REQUEST.get('folderfilter', '')
filter = context.decodeFolderFilter(folderfilter)
items = context.listFolderContents(contentFilter=filter)
items = sequence.sort( items, ((key, 'cmp', reverse and 'desc' or 'asc'),) )
batch_obj = Batch(items, 25, b_start, orphan=0)
items = []
i = 1
for item in batch_obj:
    item_icon = item.getIcon(1)
    item_id = item.getId()
    item_position = key == 'position' and str(b_start + i) or '...'
    i += 1
    item_url = item.getActionInfo( ('object/folderContents',
                                    'object/view') )['url']
    items.append( { 'checkbox': items_manage_allowed and
                                ('cb_%s' % item_id) or '',
                    'icon': item_icon and
                            ( '%s/%s' % (portal_url, item_icon) ) or '',
                    'id': item_id,
                    'modified': item.ModificationDate(),
                    'position': item_position,
                    'title': item.Title(),
                    'type': item.Type() or None,
                    'url': item_url } )
navigation = context.getBatchNavigation(batch_obj, target, **default_kw)
options['batch'] = { 'listColumnInfos': tuple(columns),
                     'listItemInfos': tuple(items),
                     'navigation': navigation }

hidden_vars = []
for name, value in html_marshal(**default_kw):
    hidden_vars.append( {'name': name, 'value': value} )
buttons = []
if items_manage_allowed:
    if items_add_allowed and context.allowedContentTypes():
        buttons.append( {'name': 'items_new', 'value': _(u'New...')} )
        if items:
            buttons.append( {'name': 'items_rename',
                             'value': _(u'Rename...')} )
    if items:
        buttons.append( {'name': 'items_cut', 'value': _(u'Cut')} )
        buttons.append( {'name': 'items_copy', 'value': _(u'Copy')} )
    if items_add_allowed and context.cb_dataValid():
        buttons.append( {'name': 'items_paste', 'value': _(u'Paste')} )
    if items_delete_allowed and items:
        buttons.append( {'name': 'items_delete', 'value': _(u'Delete')} )
length = batch_obj.sequence_length
is_orderable = items_move_allowed and (key == 'position') and length > 1
is_sortable = items_move_allowed and not is_default
deltas = range( 1, min(5, length) ) + range(5, length, 5)
options['form'] = { 'action': target,
                    'listHiddenVarInfos': tuple(hidden_vars),
                    'listButtonInfos': tuple(buttons),
                    'listDeltas': tuple(deltas),
                    'is_orderable': is_orderable,
                    'is_sortable': is_sortable }

return context.folder_contents_template(**decode(options, script))



##parameters=remote_url, **kw
##
from Products.CMFDefault.exceptions import ResourceLockedError
from Products.CMFDefault.utils import Message as _

if remote_url != context.remote_url:
    try:
        context.edit(remote_url=remote_url)
        return context.setStatus(True, _(u'Link changed.'))
    except ResourceLockedError, errmsg:
        return context.setStatus(False, errmsg)
else:
    return context.setStatus(False, _(u'Nothing to change.'))
